/**
 * Part of the diploma thesis of Thomas Everding.
 * @author Thomas Everding
 */

package de.ifgi.lehre.thesisEverding.eml;

import net.opengis.sensorML.x101.AbstractProcessType;





/**
 * Abstract super class of all EML processes and process chains.
 * 
 * @author Thomas Everding
 * 
 */
public abstract class AEMLProcess {
	
	protected IEMLProcessParent parent;
	
	/**
	 * Used to fully reference to the in- / outputs and parameters from outside (parent).
	 */
	protected String referencePrefix;
	
	
	/**
	 * Initializes this process.
	 * 
	 * @param aProcess SensorML description of this process.
	 */
	public abstract void initialize(AbstractProcessType aProcess);
	

	/**
	 * Handles a received input value.
	 * 
	 * @param inputName the name of the input or parameter (destination name)
	 * @param value the value of the input or parameter
	 */
	public abstract void handleInput(String inputName, Object value);
	

	/**
	 * stops the processing
	 */
	public abstract void stop();
	

	/**
	 * sets the parent of this EML process
	 * 
	 * @param parent the parent to set
	 */
	public void setParent(IEMLProcessParent parent) {
		this.parent = parent;
	}
	

	/**
	 * Sets the reference prefix. Used to fully reference to the in- / outputs and parameters from outside.
	 * 
	 * @param referencePrefix the referencePrefix to set
	 */
	public void setReferencePrefix(String referencePrefix) {
		this.referencePrefix = referencePrefix;
	}
	
}
