/**
 * Part of the diploma thesis of Thomas Everding.
 * @author Thomas Everding
 */

package de.ifgi.lehre.thesisEverding.eml;

import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;

import javax.swing.*;

/**
 * Dialog for the configuration of the EML process plugin.
 * 
 * @author Thomas Everding
 *
 */
public class EMLConfigDialog extends JDialog implements ActionListener{

	/**
	 * generated serial version UID
	 */
	private static final long serialVersionUID = 2743946913993095399L;
	
	private EMLProcessPlugin plugin;
	
	private JTextField tfPath;
	private JButton btBrowsePath;
	private JPanel pnlPath;
	
	private JTextField tfName;
	private JLabel lblName;
	private JPanel pnlName;
	
	private JButton btOK;

	private File selectedFile;
	
	
	/**
	 * 
	 * Constructor
	 *
	 * @param plugin the plugin that should get configured
	 */
	public EMLConfigDialog (EMLProcessPlugin plugin) {
		super();
		
		this.plugin = plugin;
		
		this.buildGUI();
	}
	
	
	/**
	 * builds the frame
	 */
	private void buildGUI() {
		//path text field
		tfPath = new JTextField();
		tfPath.setEditable(false);
		tfPath.setBounds(0, 0, 300, 20);
		
		//path browse button
		btBrowsePath = new JButton();
		btBrowsePath.setText("Browse...");
		btBrowsePath.setBounds(0, 30, 100, 20);
		btBrowsePath.addActionListener(this);
		
		//path panel
		pnlPath = new JPanel();
		pnlPath.setLayout(null);
		pnlPath.add(tfPath);
		pnlPath.add(btBrowsePath);
		pnlPath.setBounds(5, 5, 300, 50);
		
		//OK button
		btOK = new JButton();
		btOK.setText("OK");
		btOK.addActionListener(this);
		
		//name
		tfName = new JTextField();
		tfName.setText(plugin.getName());
		tfName.setPreferredSize(new Dimension(110, 20));
		
		lblName = new JLabel();
		lblName.setText("process name:");
		
		pnlName = new JPanel();
		pnlName.add(lblName);
		pnlName.add(tfName);
		
		//content pane
		this.getContentPane().setLayout(new BorderLayout());
		
		this.getContentPane().add(pnlName, BorderLayout.NORTH);
		this.getContentPane().add(pnlPath, BorderLayout.CENTER);
		this.getContentPane().add(btOK, BorderLayout.SOUTH);
		
		this.setTitle("configure " + plugin.getName());
		this.setSize(318, 140);
	}
	
	
	/**
	 * Shows or hides the frame depending on parameter visible.
	 * (calls super.setVisible and saves configuration data)
	 * @param visible true for visible
	 */
	@Override
	public void setVisible(boolean visible) {
		if (!visible) {
			//submit data to plugin
			this.submitConfigData();
		}
		super.setVisible(visible);
	}
	
	
	/**
	 * stores the config data in the plugin
	 */
	private void submitConfigData() {
		//submit name
		if (!tfName.getText().equals("")) {
			plugin.setName(tfName.getText());
		}
		
		//submit SensorML file
		plugin.loadSensorMLFile(selectedFile);
	}
	
	
	/**
	 * lets the user select a file using a FileDialog
	 */
	private void selectFile() {
		//build file dialog
		FileDialog fd = new FileDialog(this);
		fd.setTitle("Select a valid SensorML file");
		fd.setDirectory(tfPath.getText());
		
		//show the dialog
		fd.setVisible(true);
		
		if (fd.getFile() != null) {
			selectedFile = new File(fd.getDirectory() + File.separator + fd.getFile());
			tfPath.setText(selectedFile.getAbsolutePath());
		}
	}


	public void actionPerformed(ActionEvent e) {
		if (e.getSource().equals(btOK)) {
			//OK pressed
			this.setVisible(false);
		}
		else if (e.getSource().equals(btBrowsePath)){
			//browse pressed
			this.selectFile();
		}
	}
}
