/**
 * Part of the diploma thesis of Thomas Everding.
 * @author Thomas Everding
 */

package de.ifgi.lehre.thesisEverding.eml;

import java.io.File;
import java.util.logging.Logger;

import net.opengis.sensorML.x101.AbstractProcessType;
import net.opengis.sensorML.x101.SensorMLDocument;
import de.ifgi.lehre.thesisEverding.process.AProcess;
import de.ifgi.lehre.thesisEverding.sml.SensorMLParsingHelper;

/**
 * Process to execute EML using esper
 * 
 * @author Thomas Everding
 *
 */
public class EMLProcessPlugin extends AProcess{
	
	/*
	 * Logger instance for this class
	 */
	private Logger logger = Logger.getLogger(this.getClass().getName());
	
	private String name = "EML Process";
	
	private boolean ready = false;
	
	
	/**
	 * 
	 * Constructor
	 *
	 */
	public EMLProcessPlugin() {
		this.configGUI = new EMLConfigDialog(this);
	}
	

	@Override
	public String getName() {
		return name;
	}
	

	@Override
	public boolean isReady() {
		return ready;
	}
	

	@Override
	public void setName(String newName) {
		this.name = newName;
	}

	
	/**
	 * loads a given SensorML file
	 * 
	 * @param sensorMLFile
	 */
	public void loadSensorMLFile(File sensorMLFile) {
		//check file
		if (sensorMLFile == null) {
			logger.fine("no file submitted");
			ready = false;
			return;
		}
		if (!sensorMLFile.exists()) {
			logger.fine("can not find '" + sensorMLFile.getAbsolutePath() + "'");
			ready = false;
			return;
		}
		
		try {
			/*
			 * parse SensorML document
			 */
			SensorMLDocument document;
			
			try {
				document = SensorMLDocument.Factory.parse(sensorMLFile);
			}
			catch (Throwable t) {
				logger.severe("unable to parse the given file:"+ t);
				ready = false;
				return;
			}
			
			//build parsing helper
			SensorMLParsingHelper smlParser = new SensorMLParsingHelper(document);
			
			//get process
			AbstractProcessType aProcess = smlParser.getAbstractProcess();
			
			//load inputs
			this.inputDescriptions = smlParser.parseInputDescriptions();
			
			//load parameters
			this.parameterDescriptions = smlParser.parseParameterDescriptions();
			
			//load outputs
			this.outputDescriptions = smlParser.parseOutputDescriptions();
			
			//check parsing
			if (smlParser.isInvalidDocument()) {
				ready = false;
				return;
			}
			
			/*
			 * create and configure new executor
			 */
			EMLProcessExecutor exe = new EMLProcessExecutor(this, aProcess);
			
			this.executor = exe;
		}
		catch (Exception ex) {
			logger.severe("an exception occurred while parsing the SensorML document:"+ ex);
			ready = false;
			return;
		}
		ready = true;
	}
	
}
