/**
 * Part of the diploma thesis of Thomas Everding.
 * @author Thomas Everding
 */

package de.ifgi.lehre.thesisEverding.eml.esper;

import java.util.HashMap;
import java.util.Vector;

import com.espertech.esper.client.UpdateListener;
import com.espertech.esper.event.EventBean;

import de.ifgi.lehre.thesisEverding.eml.Constants;
import de.ifgi.lehre.thesisEverding.eml.event.MapEvent;

/**
 * Listener for the counting statement of repetitive patterns.
 * 
 * @author Thomas Everding
 *
 */
public class CountingListener implements UpdateListener{
	
	private EsperController controller;
	
	private String inputEventName;

	private String eventName;
	
	
	/**
	 * 
	 * Constructor
	 *
	 * @param controller the esper controller
	 * @param inputEventName name of the event which is counted
	 */
	public CountingListener(EsperController controller, String inputEventName) {
		this.controller = controller;
		this.inputEventName = inputEventName;
		
		this.initialize();
	}

	
	/**
	 * initializes this listener
	 */
	private void initialize() {
		//register counting event at esper engine
		HashMap<String, Object> eventProperties = new HashMap<String, Object>();
		eventProperties.put(MapEvent.START_KEY, Long.class);
		eventProperties.put(MapEvent.END_KEY, Long.class);
		eventProperties.put(MapEvent.CAUSALITY_KEY, Vector.class);
		
		eventName = inputEventName + Constants.REPETIVITE_COUNT_EVENT_SUFFIX;
		this.controller.registerEvent(eventName, eventProperties);
	}

	public void update(EventBean[] newEvents, EventBean[] oldEvents) {
		if (newEvents == null) {
			//no new events
			return;
		}
		
		//handle all events
		for (EventBean bean : newEvents) {
			this.handleEvent(bean);
		}
	}
	
	
	/**
	 * handles a single new event
	 * 
	 * @param bean the new event
	 */
	private synchronized void handleEvent(EventBean bean) {
		//create new event, property values are regardless
		MapEvent event = new MapEvent(1, 1);
		
		//send event
		this.controller.sendEvent(eventName, event);
	}


	/**
	 * @return the inputEventName
	 */
	public String getInputEventName() {
		return this.inputEventName;
	}
	
}
