/**
 * Part of the diploma thesis of Thomas Everding.
 * @author Thomas Everding
 */

package de.ifgi.lehre.thesisEverding.eml.esper;

import java.util.logging.Logger;

import de.ifgi.lehre.thesisEverding.eml.constants.SupportedDataTypes;

/**
 * Converts strings from the {@link SupportedDataTypes} to {@link Class} objects.
 * 
 * @author Thomas Everding
 *
 */
public class DataTypeNameToClassConverter {
	
	/*
	 * Logger instance for this class
	 */
	private static Logger logger = Logger.getLogger("DataTypeNameToClassConverter");
	
	
	/**
	 * converts data types from string to class
	 * 
	 * @param dataTypeString data type as string
	 * @return data type as class
	 */
	@SuppressWarnings("unchecked")
	public static synchronized Class convert(String dataTypeString) {
		if (dataTypeString.equals(SupportedDataTypes.BOOLEAN)) {
			return Boolean.class;
		}
		
		if (dataTypeString.equals(SupportedDataTypes.CATEGORY)) {
			return String.class;
		}
		
		if (dataTypeString.equals(SupportedDataTypes.COUNT)) {
			return Integer.class;
		}
		
		if (dataTypeString.equals(SupportedDataTypes.EVENT)) {
			//TODO
			return null;
		}
		
		if (dataTypeString.equals(SupportedDataTypes.QUANTITY)) {
			return Double.class;
		}
		
		if (dataTypeString.equals(SupportedDataTypes.TEXT)) {
			return String.class;
		}
		
		if (dataTypeString.equals(SupportedDataTypes.TIME)) {
			return Long.class;
		}
		
		//data type not supported
		logger.severe("data type '" + dataTypeString + "' is not supported");
		return null;
	}
}
