/**
 * Part of the diploma thesis of Thomas Everding.
 * @author Thomas Everding
 */

package de.ifgi.lehre.thesisEverding.eml.esper.customFunctions;

import java.util.Vector;

import de.ifgi.lehre.thesisEverding.eml.event.MapEvent;

/**
 * provides methods to perform causality test
 * 
 * @author Thomas Everding
 *
 */
public class CausalityMethods {
	
	/**
	 * Checks if an event is a causal ancestor of another event
	 * 
	 * @param event the possible ancestor
	 * @param causalVector the causal vector of the other event
	 * 
	 * @return <code>true</code> if the event is a causal ancestor
	 */
	@SuppressWarnings("unchecked")
	public static boolean isCausalAncestorOf(Object event, Object causalVector) {
		Vector<Object> vec = (Vector<Object>) causalVector;
		
		MapEvent eventMap = (MapEvent) event;
		String id = eventMap.get(MapEvent.SENSORID_KEY).toString() + eventMap.get(MapEvent.START_KEY).toString();
		
		String testID;
		MapEvent e;
		for (Object o : vec) {
			e = (MapEvent) o;
			testID = e.get(MapEvent.SENSORID_KEY).toString() + e.get(MapEvent.START_KEY).toString();
			
			if (testID.equals(id)) {
				return true;
			}
		}

		return false;
	}
	
	
	/**
	 * Checks if an event is a causal ancestor of another event
	 * 
	 * @param event the possible ancestor
	 * @param causalVector the causal vector of the other event
	 * 
	 * @return <code>true</code> if the event is not a causal ancestor
	 */
	public static boolean isNotCausalAncestorOf(Object event, Object causalVector) {
		return !isCausalAncestorOf(event, causalVector);
	}
}
