package de.ifgi.lehre.thesisEverding.eml.event;

import java.util.HashMap;

/**
 *
 * Class for holding the data types of any registered
 * phenomenon.
 *
 */
public class DataTypesMap {
	
	private static DataTypesMap instance;
	
	private HashMap<String, Object> types;
	
	private DataTypesMap() {
		types = new HashMap<String, Object>();
	}
	
	public static DataTypesMap getInstance() {
		if (instance == null) {
			instance = new DataTypesMap();
		}
		return instance;
	}
	
	/**
	 * Registers a data type for the given phenomenon.
	 * Should be called for each RegisterPublisher (new sensor)
	 * request. 
	 * 
	 * @param phenomenon Phenomenon as String (e.g. a urn)
	 * @param type Data type (e.g. Double.class)
	 * @return true if phenomenon was not registered before. false else.
	 */
	public synchronized boolean registerNewDataType(String phenomenon, Object type) {
		if (types.containsKey(phenomenon)) {
			return false;
		}
		types.put(phenomenon, type);
		return true;
	}
	
	public synchronized Object getDataType(String phenomenon) {
		if (!types.containsKey(phenomenon)) {
			return Double.class;
		}
		return types.get(phenomenon);
	}

}
