/**
 * Part of the diploma thesis of Thomas Everding.
 * @author Thomas Everding
 */

package de.ifgi.lehre.thesisEverding.eml.filter;

import java.util.HashSet;
import java.util.logging.Logger;

import net.opengis.ogc.FilterType;
import de.ifgi.lehre.thesisEverding.eml.filter.comparison.AComparisonFilter;
import de.ifgi.lehre.thesisEverding.eml.filter.logic.ALogicFilter;
import de.ifgi.lehre.thesisEverding.eml.filter.spatial.ASpatialFilter;

/**
 * Representation of a filter. (a complete guard)
 * 
 * @author Thomas Everding
 * 
 */
public class StatementFilter implements IFilterElement {

	/*
	 * Logger instance for this class
	 */
	private Logger logger = Logger.getLogger(this.getClass().getName());

	private IFilterElement child;

	protected String usedProperty = null;


	/**
	 * 
	 * Constructor
	 * 
	 * @param filter
	 */
	public StatementFilter(FilterType filter, HashSet<Object> propertyNames) {
		this.initialize(filter, propertyNames);
	}

	/**
	 * initializes the filter
	 * 
	 * @param filter
	 *            Filter definition
	 */
	private void initialize(FilterType filter, HashSet<Object> propertyNames) {
		if (filter.isSetLogicOps()) {
			// parse logic operator
			this.child = ALogicFilter.FACTORY.buildLogicFilter(filter
					.getLogicOps(), propertyNames);
		} else if (filter.isSetComparisonOps()) {
			// parse comparison operator
			this.child = AComparisonFilter.FACTORY.buildComparisonFilter(filter
					.getComparisonOps(), propertyNames);
		} else if (filter.isSetSpatialOps()) {
			//parse spatial filter
			this.child = ASpatialFilter.FACTORY.buildSpatialFilter(filter.getSpatialOps());
		} else {
			logger.severe("operator type not supported");
			return;
		}
	}

	public String createExpressionString(boolean complexPatternGuard) {
		return this.child.createExpressionString(complexPatternGuard);
	}
	
	public void setUsedProperty(String nodeValue) {
		this.usedProperty = nodeValue;
	}

}
