/**
 * Part of the diploma thesis of Thomas Everding.
 * @author Thomas Everding
 */

package de.ifgi.lehre.thesisEverding.eml.filter.comparison;

import java.util.HashSet;

import net.opengis.ogc.PropertyIsBetweenType;
import de.ifgi.lehre.thesisEverding.eml.esper.customFunctions.MethodNames;
import de.ifgi.lehre.thesisEverding.eml.filter.expression.AFilterExpression;

/**
 * Filter to compare a value and a range.
 * 
 * @author Thomas Everding
 *
 */
public class BetweenFilter extends AComparisonFilter{
	
	private AFilterExpression lower;
	
	private AFilterExpression upper;
	
	private AFilterExpression test;
	
	
	/**
	 * 
	 * Constructor
	 *
	 * @param betweenOp filter definition
	 */
	public BetweenFilter(PropertyIsBetweenType betweenOp, HashSet<Object > propertyNames) {
		this.test  = AFilterExpression.FACTORY.buildFilterExpression(betweenOp.getExpression(), propertyNames, this);
		this.lower = AFilterExpression.FACTORY.buildFilterExpression(betweenOp.getLowerBoundary().getExpression(), propertyNames, this);
		this.upper = AFilterExpression.FACTORY.buildFilterExpression(betweenOp.getUpperBoundary().getExpression(), propertyNames, this);
	}

	public String createExpressionString(boolean complexPatternGuard) {
		String result = "";
		
		if (lower.getUsedProperty() != null || upper.getUsedProperty() != null || test.getUsedProperty() != null) {
			result += "(";
			
			boolean first = true;
			
			if (lower.getUsedProperty() != null) {
				result += MethodNames.PROPERTY_EXISTS_NAME + "(this, \"" + lower.getUsedProperty() + "\") ";
				first = false;
			}
			
			if (upper.getUsedProperty() != null) {
				
				if (!first) {
					result += "AND ";
				}
				
				result += MethodNames.PROPERTY_EXISTS_NAME + "(this, \"" + upper.getUsedProperty() + "\") ";
				first = false;
			}
			
			if (test.getUsedProperty() != null) {
				
				if (!first) {
					result += "AND ";
				}
				
				result += MethodNames.PROPERTY_EXISTS_NAME + "(this, \"" + test.getUsedProperty() + "\") ";
				first = false;
			}
			
			result += ") AND ";
		}
		
		result += "("
						+ this.test.createExpressionString(complexPatternGuard)
						+ " between "
						+ this.lower.createExpressionString(complexPatternGuard)
						+ " and "
						+ this.upper.createExpressionString(complexPatternGuard)
						+ ")";
		return result;
	}
}
