/**
 * Part of the diploma thesis of Thomas Everding.
 * @author Thomas Everding
 */
package de.ifgi.lehre.thesisEverding.eml.filter.comparison;

import java.util.HashSet;
import java.util.logging.Logger;

import javax.xml.namespace.QName;

import net.opengis.ogc.*;
import de.ifgi.lehre.thesisEverding.eml.filter.logic.ALogicFilter;

/**
 * Builds comparison filters.
 * 
 * @author Thomas Everding
 *
 */
public class ComparisonFilterFactory {
	
	/*
	 * Logger instance for this class
	 */
	private Logger logger = Logger.getLogger(this.getClass().getName());
	
	private static final QName BETWEEN_QNAME = new QName("http://www.opengis.net/ogc", "PropertyIsBetween");
	
	private static final QName EQUAL_QNAME = new QName("http://www.opengis.net/ogc", "PropertyIsEqualTo");
	
	private static final QName GREATER_QNAME = new QName("http://www.opengis.net/ogc", "PropertyIsGreaterThan");
	
	private static final QName GREATER_OR_EQUAL_QNAME = new QName("http://www.opengis.net/ogc", "PropertyIsGreaterThanOrEqualTo");
	
	private static final QName LESS_QNAME = new QName("http://www.opengis.net/ogc", "PropertyIsLessThan");
	
	private static final QName LESS_OR_EQUAL_QNAME = new QName("http://www.opengis.net/ogc", "PropertyIsLessThanOrEqualTo");
	
	private static final QName NOT_EQUAL_QNAME = new QName("http://www.opengis.net/ogc", "PropertyIsNotEqualTo");
	
	/**
	 * Builds a new comparison filter
	 * 
	 * @param comparisonOp definition of the filter
	 * 
	 * @return the new {@link ALogicFilter}
	 */
	public AComparisonFilter buildComparisonFilter(ComparisonOpsType comparisonOp, HashSet<Object > propertyNames) {
		//TODO
		
		QName coQName = comparisonOp.newCursor().getName();
		
		/*
		 * non binary operators
		 */
		
		//check between
		if (BETWEEN_QNAME.equals(coQName)) {
			PropertyIsBetweenType betweenOp = (PropertyIsBetweenType) comparisonOp;
			return new BetweenFilter(betweenOp, propertyNames);
		}
		
		/*
		 * binary operators
		 */
		BinaryComparisonOpType binaryOp = (BinaryComparisonOpType) comparisonOp;
		
		//check equal
		if (EQUAL_QNAME.equals(coQName)) {
			return new EqualToFilter(binaryOp, propertyNames);
		}
		
		//check not equal
		else if (NOT_EQUAL_QNAME.equals(coQName)) {
			return new NotEqualToFilter(binaryOp, propertyNames);
		}
		
		//check greater
		else if (GREATER_QNAME.equals(coQName)) {
			return new GreaterThanFilter(binaryOp, propertyNames);
		}
		
		//check greater or equal
		else if (GREATER_OR_EQUAL_QNAME.equals(coQName)) {
			return new GreaterThanOrEqualToFilter(binaryOp, propertyNames);
		}
		
		//check less
		else if (LESS_QNAME.equals(coQName)) {
			return new LessThanFilter(binaryOp, propertyNames);
		}
		
		//check less or equal
		else if (LESS_OR_EQUAL_QNAME.equals(coQName)) {
			return new LessThanOrEqualToFilter(binaryOp, propertyNames);
		}
		
		logger.severe("unable to build comparison filter for '" + comparisonOp.newCursor().getName().toString() + "'");
		return null;
	}
	
}
