/**
 * Part of the diploma thesis of Thomas Everding.
 * @author Thomas Everding
 */

package de.ifgi.lehre.thesisEverding.eml.filter.expression;

import java.util.HashSet;
import java.util.logging.Logger;

import javax.xml.namespace.QName;

import net.opengis.ogc.*;
import de.ifgi.lehre.thesisEverding.eml.filter.IFilterElement;

/**
 * Builds {@link AFilterExpression}s from given {@link ExpressionType}s.
 * 
 * @author Thomas Everding
 *
 */
public class FilterExpressionFactory {
	
	/*
	 * Logger instance for this class
	 */
	private Logger logger = Logger.getLogger(this.getClass().getName());
	
	private static final QName ADD_QNAME = new QName("http://www.opengis.net/ogc", "Add");
	
	private static final QName DIV_QNAME = new QName("http://www.opengis.net/ogc", "Div");
	
	private static final QName LITERAL_QNAME = new QName("http://www.opengis.net/ogc", "Literal");
	
	private static final QName MUL_QNAME = new QName("http://www.opengis.net/ogc", "Mul");
	
	private static final QName PROPERTY_NAME_QNAME = new QName("http://www.opengis.net/ogc", "PropertyName");
	
	private static final QName SUB_QNAME = new QName("http://www.opengis.net/ogc", "Sub");
	
	/**
	 * Parses an {@link ExpressionType}.
	 * 
	 * @param expression the expression to parse
	 * @return the parsed expression
	 */
	public AFilterExpression buildFilterExpression (ExpressionType expression, HashSet<Object> propertyNames, IFilterElement parent) {
		QName exprQName = expression.newCursor().getName();
		
		/*
		 * non binary expression 
		 */
		
		//check literal
		if (LITERAL_QNAME.equals(exprQName)) {
			LiteralType lt = (LiteralType) expression;
			return new LiteralExpression(lt);
		}
		
		//check PropertyName
		else if (PROPERTY_NAME_QNAME.equals(exprQName)) {
			PropertyNameType pnt = (PropertyNameType) expression;
			
			
			PropertyNameExpression result = new PropertyNameExpression(pnt, propertyNames);
			//replaceAll(":", "__")
			result.setUsedProperty(pnt.getDomNode().getFirstChild().getNodeValue());
			return result;
		}
		
		/*
		 * binary expressions
		 */
		BinaryOperatorType binaryOP = (BinaryOperatorType) expression;
		
		//check Add
		if (ADD_QNAME.equals(exprQName)) {
			//create new AddExpression
			return new AddExpression(binaryOP, propertyNames);
		}
		
		//check Div
		else if (DIV_QNAME.equals(exprQName)) {
			//create new DivExpression
			return new DivExpression(binaryOP, propertyNames);
		}
		
		//check Mul
		else if (MUL_QNAME.equals(exprQName)) {
			//create new MulExpression
			return new MulExpression(binaryOP, propertyNames);
		}
		
		//check Sub
		else if (SUB_QNAME.equals(exprQName)) {
			//create new SubExpression
			return new SubExpression(binaryOP, propertyNames);
		}
		
		logger.severe("unable to build filter expression for '" + exprQName.toString() + "'");
		return null;
	}
	
}
