/**
 * Part of the diploma thesis of Thomas Everding.
 * @author Thomas Everding
 */

package de.ifgi.lehre.thesisEverding.eml.filter.logic;

import java.util.HashSet;

import net.opengis.ogc.BinaryLogicOpType;
import de.ifgi.lehre.thesisEverding.eml.filter.IFilterElement;
import de.ifgi.lehre.thesisEverding.eml.filter.comparison.AComparisonFilter;
import de.ifgi.lehre.thesisEverding.eml.filter.spatial.ASpatialFilter;

/**
 * Representation of binary logic filters.
 * 
 * @author Thomas Everding
 *
 */
public abstract class ABinaryLogicFilter extends ALogicFilter {
	
	protected IFilterElement first;
	
	protected IFilterElement second;
	
	/**
	 * initializes the filter
	 * 
	 * @param binaryOp the filter definition
	 */
	protected void initialize(BinaryLogicOpType binaryOp, HashSet<Object > propertyNames) {
		if (binaryOp.getLogicOpsArray().length == 2) {
			//only logical operators
			first  = ALogicFilter.FACTORY.buildLogicFilter(binaryOp.getLogicOpsArray(0), propertyNames);
			second = ALogicFilter.FACTORY.buildLogicFilter(binaryOp.getLogicOpsArray(1), propertyNames);
		}
		else if (binaryOp.getComparisonOpsArray().length == 2){
			//only comparison operators
			first  = AComparisonFilter.FACTORY.buildComparisonFilter(binaryOp.getComparisonOpsArray(0), propertyNames);
			second = AComparisonFilter.FACTORY.buildComparisonFilter(binaryOp.getComparisonOpsArray(1), propertyNames);
		}
		else if (binaryOp.getSpatialOpsArray().length == 2) {
			first  = ASpatialFilter.FACTORY.buildSpatialFilter(binaryOp.getSpatialOpsArray(0));
			second = ASpatialFilter.FACTORY.buildSpatialFilter(binaryOp.getSpatialOpsArray(1));
		}
		else {
			if (binaryOp.getLogicOpsArray().length == 1) {
				first  = ALogicFilter.FACTORY.buildLogicFilter(binaryOp.getLogicOpsArray(0), propertyNames);
			}
			if (binaryOp.getComparisonOpsArray().length == 1) {
				if (first != null) {
					second = AComparisonFilter.FACTORY.buildComparisonFilter(binaryOp.getComparisonOpsArray(0), propertyNames);
				} else {
					first = AComparisonFilter.FACTORY.buildComparisonFilter(binaryOp.getComparisonOpsArray(0), propertyNames);
				}
			}
			if (binaryOp.getSpatialOpsArray().length == 1) {
				if (first != null) {
					if (second == null) {
						second = ASpatialFilter.FACTORY.buildSpatialFilter(binaryOp.getSpatialOpsArray(0));
					}
				} else {
					first = ASpatialFilter.FACTORY.buildSpatialFilter(binaryOp.getSpatialOpsArray(0));
				}
			}
//			//one of each
//			first  = ALogicFilter.FACTORY.buildLogicFilter(binaryOp.getLogicOpsArray(0), propertyNames);
//			second = AComparisonFilter.FACTORY.buildComparisonFilter(binaryOp.getComparisonOpsArray(0), propertyNames);
		}
	}
}
