/**
 * Part of the diploma thesis of Thomas Everding.
 * @author Thomas Everding
 */

package de.ifgi.lehre.thesisEverding.eml.filter.logic;

import java.util.HashSet;
import java.util.logging.Logger;

import javax.xml.namespace.QName;

import net.opengis.ogc.*;

/**
 * Builds logic filters.
 * 
 * @author Thomas Everding
 *
 */
public class LogicFilterFactory {
	
	/*
	 * Logger instance for this class
	 */
	private Logger logger = Logger.getLogger(this.getClass().getName());
	
	private static final QName AND_QNAME = new QName("http://www.opengis.net/ogc", "And");
	
	private static final QName OR_QNAME = new QName("http://www.opengis.net/ogc", "Or");
	
	private static final QName NOT_QNAME = new QName("http://www.opengis.net/ogc", "Not");
	
	/**
	 * Builds a new logic filter
	 * 
	 * @param logicOp definition of the filter
	 * 
	 * @return the new {@link ALogicFilter}
	 */
	public ALogicFilter buildLogicFilter(LogicOpsType logicOp, HashSet<Object > propertyNames) {
		//TODO
		QName loQName = logicOp.newCursor().getName();
		
		/*
		 * non binary operators 
		 */
		
		//check Not
		if (NOT_QNAME.equals(loQName)) {
			//create new NotFilter
			UnaryLogicOpType unaryOp = (UnaryLogicOpType) logicOp;
			return new NotFilter(unaryOp, propertyNames);
		}
		
		/*
		 * binary operators
		 */
		BinaryLogicOpType binaryOp = (BinaryLogicOpType) logicOp;
		
		//check And
		if (AND_QNAME.equals(loQName)) {
			//create new AndFilter
			return new AndFilter(binaryOp, propertyNames);
		}
		
		//check Or
		else if (OR_QNAME.equals(loQName)) {
			//create new OrFilter
			return new OrFilter(binaryOp, propertyNames);
		}
		
		logger.severe("unable to build filter expression for '" + loQName.toString() + "'");
		return null;
	}
	
}
