package de.ifgi.lehre.thesisEverding.eml.filter.spatial;

import net.opengis.gml.EnvelopeType;
import net.opengis.gml.PointType;

import org.apache.muse.util.xml.XmlUtils;
import org.w3c.dom.Element;

import com.vividsolutions.jts.geom.*;
import com.vividsolutions.jts.io.ParseException;
import com.vividsolutions.jts.io.WKTReader;

import de.ifgi.lehre.thesisEverding.eml.filter.IFilterElement;


/**
 * 
 * @author Matthes Rieke <m.rieke@uni-muenster.de>
 *
 */
public abstract class ASpatialFilter implements IFilterElement {
	
	/**
	 * Factory to build new comparison filters.
	 */
	public static final SpatialFilterFactory FACTORY = new SpatialFilterFactory();
	
	/**
	 * creates a Geometry Object from an EnvelopeType.
	 * @throws ParseException if the geometry could not be parsed.
	 */
	protected final Geometry getGeometryFromEnvelope(EnvelopeType et) throws ParseException {
		//get upper corner and split at " "
		Element elem = (Element) et.getUpperCorner().getDomNode();
		String ucString = XmlUtils.toString(elem.getFirstChild());
		String[] uc = ucString.split(" ");
		
		int i = 0;
		for (String string : uc) {
			string = string.trim();
			uc[i] = string;
			i++;
		}

		//get lower corner and split at " "
		elem = (Element) et.getLowerCorner().getDomNode();
		String lcString = XmlUtils.toString(elem.getFirstChild());
		String[] lc = lcString.split(" ");

		i = 0;
		for (String string : lc) {
			string = string.trim();
			lc[i] = string;
			i++;
		}
		
		String wktString = "POLYGON(("+ lc[0] +" "+ lc[1] +", "+ lc[0] +" "+ uc[1] +", "+ uc[0] +" "+ uc[1] +
				", "+ uc[0] +" "+ lc[1] +", "+ lc[0] +" "+ lc[1]+ "))";
		
		WKTReader wktReader = new WKTReader();
		return wktReader.read(wktString);
	}
	
	protected final Geometry getGeometryFromPoint(PointType pt) {
		GeometryFactory gf = new GeometryFactory();
		Coordinate c = new Coordinate(pt.getCoord().getX().doubleValue(),
				pt.getCoord().getY().doubleValue(), pt.getCoord().getZ().doubleValue());
		
		return gf.createPoint(c);
	}

}
