package de.ifgi.lehre.thesisEverding.eml.filter.spatial;

import javax.xml.namespace.QName;

import net.opengis.ogc.*;

/**
 * 
 * @author Matthes Rieke <m.rieke@uni-muenster.de>
 *
 */
public class SpatialFilterFactory {
	/*
	 * BBox QName
	 */
	private static final QName BBOX_QNAME = new QName("http://www.opengis.net/ogc", "BBOX");
	
	/*
	 * Distance Buffer Ops QNames
	 */
	private static final QName BEYOND_QNAME = new QName("http://www.opengis.net/ogc", "Beyond");
	
	private static final QName DWITHIN_QNAME = new QName("http://www.opengis.net/ogc", "DWithin");
	
	/*
	 * Binary Spatial Ops QNames
	 */
	private static final QName CONTAINS_QNAME = new QName("http://www.opengis.net/ogc", "Contains");
	
	private static final QName CROSSES_QNAME = new QName("http://www.opengis.net/ogc", "Crosses");
	
	private static final QName DISJOINT_QNAME = new QName("http://www.opengis.net/ogc", "Disjoint");
	
	private static final QName EQUALS_QNAME = new QName("http://www.opengis.net/ogc", "Equals");
	
	private static final QName INTERSECTS_QNAME = new QName("http://www.opengis.net/ogc", "Intersects");
	
	private static final QName OVERLAPS_QNAME = new QName("http://www.opengis.net/ogc", "Overlaps");
	
	private static final QName TOUCHES_QNAME = new QName("http://www.opengis.net/ogc", "Touches");
	
	private static final QName WITHIN_QNAME = new QName("http://www.opengis.net/ogc", "Within");
	
	public ASpatialFilter buildSpatialFilter(SpatialOpsType sot) {
		QName sotQName = sot.newCursor().getName();
		
		//bbox ops types
		if (sotQName.equals(BBOX_QNAME)) {
			BBOXType bboxOp = (BBOXType) sot;
			return new BBOXFilter(bboxOp);
		}
		
		//Distance Buffer Ops types
		if (sotQName.equals(BEYOND_QNAME) || sotQName.equals(DWITHIN_QNAME)) {
			DistanceBufferType dbOp = (DistanceBufferType) sot;
		
			if (BEYOND_QNAME.equals(sotQName)) {
				return new BeyondFilter(dbOp);
			}
			
			else if (DWITHIN_QNAME.equals(sotQName)) {
				return new DWithinFilter(dbOp);
			}
		}
		
		//binary spatial ops types
		BinarySpatialOpType bsOp = (BinarySpatialOpType) sot;
		
		if (CONTAINS_QNAME.equals(sotQName)) {
			return new ContainsFilter(bsOp);
		}
		
		else if (CROSSES_QNAME.equals(sotQName)) {
			return new CrossesFilter(bsOp);
		}
		
		else if (DISJOINT_QNAME.equals(sotQName)) {
			return new DisjointFilter(bsOp);
		}
		
		else if (EQUALS_QNAME.equals(sotQName)) {
			return new EqualsFilter(bsOp);
		}
		
		else if (INTERSECTS_QNAME.equals(sotQName)) {
			return new IntersectsFilter(bsOp);
		}
		
		else if (OVERLAPS_QNAME.equals(sotQName)) {
			return new OverlapsFilter(bsOp);
		}
		
		else if (TOUCHES_QNAME.equals(sotQName)) {
			return new TouchesFilter(bsOp);
		}
		
		else if (WITHIN_QNAME.equals(sotQName)) {
			return new WithinFilter(bsOp);
		}
		
		return null;
	}

}
