/**
 * Part of the diploma thesis of Thomas Everding.
 * @author Thomas Everding
 */

package de.ifgi.lehre.thesisEverding.eml.pattern;

import java.util.HashMap;
import java.util.logging.Logger;

import de.ifgi.lehre.thesisEverding.eml.Constants;


/**
 * Represents a single view
 * 
 * @author Thomas Everding
 * 
 */
public class DataView {
	
	/*
	 * Logger instance for this class
	 */
	private Logger logger = Logger.getLogger(this.getClass().getName());
	
	private String viewName;
	
	private String esperString = "";
	
	private boolean batch = false;
	
	private HashMap<String, Object> parameters;
	
	
	/**
	 * 
	 * Constructor
	 * 
	 */
	public DataView() {
		this.parameters = new HashMap<String, Object>();
	}
	

	/**
	 * @return the viewName
	 */
	public String getViewName() {
		return this.viewName;
	}
	

	/**
	 * @param viewName the viewName to set
	 */
	public void setViewName(String viewName) {
		this.viewName = viewName;
	}
	

	/**
	 * @return the parameters
	 */
	public HashMap<String, Object> getParameters() {
		return this.parameters;
	}
	

	/**
	 * adds a parameter for the view
	 * 
	 * @param pName name of the parameter
	 * @param pValue value of the parameter
	 */
	public void addParameter(String pName, Object pValue) {
		this.parameters.put(pName, pValue);
	}
	

	/**
	 * @return the batch
	 */
	public boolean isBatch() {
		return this.batch;
	}
	

	/**
	 * @param batch the batch to set
	 */
	public void setBatch(boolean batch) {
		this.batch = batch;
	}
	

	/**
	 * 
	 * @return the view as esper statement part, creates it if necessary
	 */
	public String getViewString() {
		if (this.esperString.equals("")) {
			//create esper string
			
			/*
			 * AllView
			 */
			if (this.viewName.equals(Constants.VIEW_ALL_NAME)) {
				this.esperString = ".win:keepall()";
			}
			
			/*
			 * LengthView
			 */
			else if (this.viewName.equals(Constants.VIEW_LENGTH_NAME)) {
				this.esperString = ".win:length";
				if (batch) {
					this.esperString += "_batch";
				}
				this.esperString += "(" 
								  + this.parameters.get(Constants.VIEW_PARAM_EVENT_COUNT_NAME) 
								  + ")";
			}
			
			/*
			 * TimeView
			 */
			else if (this.viewName.equals(Constants.VIEW_TIME_NAME)) {
				this.esperString = ".win:time";
				if (batch) {
					this.esperString += "_batch";
				}
				this.esperString += "(" 
								  + this.parameters.get(Constants.VIEW_PARAM_DURATION_NAME) 
								  + ")";
			}
			
			/*
			 * TimeLengthView
			 */
			else if (this.viewName.equals(Constants.VIEW_TIME_LENGTH_NAME)) {
				if (!batch) {
					logger.severe("sliding TimeLengthView is not supported, using batch mode");
				}
				this.esperString = ".win:time_length_batch("
								 + this.parameters.get(Constants.VIEW_PARAM_DURATION_NAME)
								 + ", "
								 + this.parameters.get(Constants.VIEW_PARAM_EVENT_COUNT_NAME)
								 + ")";
			}
			
			/*
			 * create statements for additional views here
			 */
		}
		return this.esperString;
	}
}
