/**
 * Part of the diploma thesis of Thomas Everding.
 * @author Thomas Everding
 */

package de.ifgi.lehre.thesisEverding.eml.pattern;

import java.util.HashMap;
import java.util.logging.Logger;

import de.ifgi.lehre.thesisEverding.eml.Constants;
import de.ifgi.lehre.thesisEverding.eml.esper.EMLParser;
import de.ifgi.lehre.thesisEverding.eml.esper.EsperController;
import de.ifgi.lehre.thesisEverding.eml.esper.customFunctions.MethodNames;
import de.ifgi.lehre.thesisEverding.eml.event.MapEvent;

/**
 * representation of a complex pattern
 * 
 * @author Thomas Everding
 * 
 */
public class PatternComplex extends AGuardedViewPattern {
	
	/*
	 * Logger instance for this class
	 */
	private Logger logger = Logger.getLogger(this.getClass().getName());
	
	private PatternOperator operator;
	
	private String firstPatternID;
	
	private String secondPatternID;
	
	private int firstSelectFunctionNumber;
	
	private int secondSelectFunctionNumber;
	
	private EsperController controller;
	
	private long maxListeningDuration = -1;

	private EMLParser parser;
	
	
	/**
	 * @return the operator
	 */
	public PatternOperator getOperator() {
		return this.operator;
	}
	

	/**
	 * sets the operator
	 * 
	 * @param operator the operator to set
	 */
	public void setOperator(PatternOperator operator) {
		this.operator = operator;
	}
	

	/**
	 * @return the first pattern ID
	 */
	public String getFirstPatternID() {
		return this.firstPatternID;
	}
	

	/**
	 * @param firstPatternID the first pattern ID to set
	 */
	public void setFirstPatternID(String firstPatternID) {
		this.firstPatternID = firstPatternID;
	}
	

	/**
	 * @return the second pattern ID
	 */
	public String getSecondPatternID() {
		return this.secondPatternID;
	}
	

	/**
	 * @param secondPatternID the second pattern ID to set
	 */
	public void setSecondPatternID(String secondPatternID) {
		this.secondPatternID = secondPatternID;
	}
	

	/**
	 * @return the firstSelectFunctionNumber
	 */
	public int getFirstSelectFunctionNumber() {
		return this.firstSelectFunctionNumber;
	}
	

	/**
	 * @param firstSelectFunctionNumber the firstSelectFunctionNumber to set
	 */
	public void setFirstSelectFunctionNumber(int firstSelectFunctionNumber) {
		this.firstSelectFunctionNumber = firstSelectFunctionNumber;
	}
	

	/**
	 * @return the secondSelectFunctionNumber
	 */
	public int getSecondSelectFunctionNumber() {
		return this.secondSelectFunctionNumber;
	}
	

	/**
	 * @param secondSelectFunctionNumber the secondSelectFunctionNumber to set
	 */
	public void setSecondSelectFunctionNumber(int secondSelectFunctionNumber) {
		this.secondSelectFunctionNumber = secondSelectFunctionNumber;
	}
	
	
	/**
	 * @param controller the controller to set
	 */
	public void setController(EsperController controller) {
		this.controller = controller;
	}
	
	
	/**
	 * @return the maxListeningDuration (ms)
	 */
	public long getMaxListeningDuration() {
		return this.maxListeningDuration;
	}


	/**
	 * @param maxListeningDuration the maxListeningDuration to set (ms)
	 */
	public void setMaxListeningDuration(long maxListeningDuration) {
		this.maxListeningDuration = maxListeningDuration;
	}
	

	@Override
	public Statement[] createEsperStatements() {
		Statement[] result = new Statement[this.selectFunctions.size()];
		
		String firstInputEventName = "";
		String secondInputEventName = "";
		if (controller != null ) {
			//get first input event name
			firstInputEventName = this.controller.getNewEventName(firstPatternID, firstSelectFunctionNumber) ;
			
			//get second input event name
			secondInputEventName = this.controller.getNewEventName(secondPatternID, secondSelectFunctionNumber);
			
		}
		
		else if (parser != null) {
			firstInputEventName = getNewEventNameWithParser(firstPatternID, firstSelectFunctionNumber) ;
			
			secondInputEventName = getNewEventNameWithParser(secondPatternID, secondSelectFunctionNumber);
		}
		
		else {
			//something went wrong
			return null;
		}
		
		/*
		 * build statements
		 */
		String selectClause;
		String fromClause;
		String whereClause = "";
		SelFunction sel;
		Statement stat;
		
		//build where clause (use maxListeningDuration and the Guard!)
		String pName = this.selectFunctions.get(0).getFunctionParameters().get(Constants.SELECT_PARAM_PROPERTY_NAME).toString();
//		String pName = this.selectFunctions.get(0).getFunctionParameters().get(Constants.FUNC_SELECT_PROPERTY_NAME).toString();
		whereClause = this.guard.createStatement(true, pName);
		
		if (this.operator.equals(Constants.OPERATOR_CAUSE_NAME)
			|| this.operator.equals(Constants.OPERATOR_BEFORE_NAME)) {
			/*
			 * CAUSE and PARALLEL have to work different (using guards with user functions)
			 */
			
			//build where clause
			if (whereClause.equals("")) {
				//no previous where clause, create one
				whereClause = Constants.EPL_WHERE
							  + " "
							  + this.buildCausalCheckString(firstInputEventName, secondInputEventName);
			}
			else {
				//append to existing where clause
				whereClause += "and"
							   + this.buildCausalCheckString(firstInputEventName, secondInputEventName);
			}
			
			//build from clause (always use and)
			fromClause = Constants.EPL_FROM
						 + " "
 						 + Constants.EPL_PATTERN
 						 + " [every ((("
 						 + firstInputEventName
 						 + "="
 						 + firstInputEventName
 						 + ") and ("
						 + secondInputEventName
						 + "="
						 + secondInputEventName
						 + ")) "
						 + this.buildPatternTimerClause()
						 + ")]";
		}
		else {
			//build from clause
			fromClause = this.buildFromClause(firstInputEventName, secondInputEventName);
		}
		
		for (int i = 0; i < result.length; i++) {
			sel = this.selectFunctions.get(i);
			
			//build select clause
			selectClause = Constants.EPL_SELECT
						   + " "
						   + sel.getSelectString(false)
						   + " ";
			
			//build statement
			stat = new Statement();
			stat.setSelectFunction(sel);
			stat.setStatement(selectClause + fromClause + whereClause);
			
			//add statement to result
			result[i] = stat;
		}
		
		return result;
	}

	
	/**
	 * Method for creating statements using just an EMLParser instead 
	 * of EsperController. (added by Matthes)
	 */
	private String getNewEventNameWithParser(String secondPatternID2,
			int selectFunctionNumber) {
		//get all patterns from parser
		HashMap<String, APattern> patterns = parser.getPatterns();
		
		//search for pattern
		if (!this.parser.getPatterns().containsKey(patternID)) {
			logger.severe("pattern ID (" + patternID + ") not found");
			return null;
		}
		APattern pattern = patterns.get(patternID);
		
		//search for select function
		if (!(pattern.getSelectFunctions().size() > selectFunctionNumber)) {
			if (!(pattern.getSelectFunctions().size() >= 0)) {
				logger.severe("No select function and therefore no 'newEventName' defined in pattern '" + patternID
						+ "'. Can not use this pattern in a repetitive pattern.");
				return null;
			}
			logger.warning("The pattern with the id '" + patternID + "does not define at least "
					+ selectFunctionNumber + " selectfunctions. Using first select function instead.");
			
			//set number to 0
			selectFunctionNumber = 0;
		}
		
		//return newEventName
		return pattern.getSelectFunctions().get(selectFunctionNumber).getNewEventName();
	}


	@Override
	public Statement[] createEsperStatements(EMLParser parser) {
		this.parser = parser;
		return createEsperStatements();
	}
	
	/**
	 * build a part of the where clause for CAUSE and PARALLEL operators
	 * 
	 * @param secondInputEventName name of the possible causal ancestor
	 * @param firstInputEventName name of the event with the causal vector
	 * 
	 * @return the part of the where clause without 'where'
	 */
	private String buildCausalCheckString(String firstInputEventName, String secondInputEventName) {
		String result = "";
		
		//add operator
		if (this.operator.equals(Constants.OPERATOR_CAUSE_NAME)) {
			//CAUSE
			result += MethodNames.IS_CAUSAL_ANCESTOR_NAME
					  + "(";
		}
		else {
			//PARALLEL
			result += MethodNames.IS_NOT_CAUSAL_ANCESTOR_NAME
			  + "(";
		}
		
		//add parameters
		result += firstInputEventName
				  + ", "
				  + secondInputEventName
				  + "."
				  + MapEvent.CAUSALITY_KEY
				  + ")";
		
		return result;
	}


	/**
	 * builds a from clause
	 * 
	 * @param firstInputEventName name of the first input event
	 * @param secondInputEventName name of the second input event
	 * @return
	 */
	private String buildFromClause(String firstInputEventName, String secondInputEventName) {
		String clause = Constants.EPL_FROM
						+ " "
						+ Constants.EPL_PATTERN
						+ " [every ((("
						+ firstInputEventName
						+ "="
						+ firstInputEventName
						+ ") "
						+ this.getOperatorString()
						+ " ("
						+ secondInputEventName
						+ "="
						+ secondInputEventName
						+ ")) "
						+ this.buildPatternTimerClause()
						+ ")]";
		return clause;
	}


	/**
	 * build the timer:within clause
	 * 
	 * @return the pattern guard (timer:within)
	 */
	private String buildPatternTimerClause() {
		if(this.maxListeningDuration < 0) {
			//no within clause
			return "";
		}
		
		//build clause
		String result = Constants.EPL_WHERE
						+ " timer:within("
						+ this.maxListeningDuration
						+ " msec)";
		return result;
	}


	/**
	 * 
	 * @return the esper representation for the operator
	 */
	private String getOperatorString() {
		String operator = "";
		
		String opName = this.operator.getName();
		
		//AND
		if (opName.equals(Constants.OPERATOR_AND_NAME)) {
			operator = "and";
		}
		
		//AND NOT
		else if (opName.equals(Constants.OPERATOR_AND_NOT_NAME)) {
			operator = ") and not (";
			/*
			 * The parentheses are necessary for correct execution.
			 * The 'every' operator must only be used for the first pattern.
			 */
		}
		
		//OR
		else if (opName.equals(Constants.OPERATOR_OR_NAME)) {
			operator = "or";
		}
		
		//BEFORE
		else if (opName.equals(Constants.OPERATOR_BEFORE_NAME)) {
			operator = "->";
		}
		
		//CAUSE
		else if (opName.equals(Constants.OPERATOR_CAUSE_NAME)) {
			operator = "and";
			//functionality must be implemented as guard using 'in'
		}
		
		//PARALLEL
		else if (opName.equals(Constants.OPERATOR_PARALLEL_NAME)) {
			operator = "and";
			//functionality must be implemented as guard using 'in'
		}
		
		//user defined operators here
		else {
			logger.severe("the operator '" + opName + "' is not supported");
		}
		
		return operator;
	}
}
