/**
 * Part of the diploma thesis of Thomas Everding.
 * @author Thomas Everding
 */

package de.ifgi.lehre.thesisEverding.eml.pattern;

import java.util.HashMap;

import de.ifgi.lehre.thesisEverding.eml.Constants;
import de.ifgi.lehre.thesisEverding.eml.esper.EMLParser;
import de.ifgi.lehre.thesisEverding.eml.esper.EsperController;

/**
 * represents a repetitive pattern
 * 
 * @author Thomas Everding
 *
 */
public class PatternRepetitive extends APattern{ 
	
	private int repetitionCount;
	
	private String patternToRepeatID;
	
	private EsperController controller;

	private int selectFunctionNumber;

	private String inputEventName;

	private EMLParser parser;
	
	/**
	 * @return the repetitionCount
	 */
	public int getRepetitionCount() {
		return this.repetitionCount;
	}

	/**
	 * @param repetitionCount the repetitionCount to set
	 */
	public void setRepetitionCount(int repetitionCount) {
		this.repetitionCount = repetitionCount;
	}

	/**
	 * @return the patternToRepeatID
	 */
	public String getPatternToRepeatID() {
		return this.patternToRepeatID;
	}

	/**
	 * @param patternToRepeatID the patternToRepeatID to set
	 */
	public void setPatternToRepeatID(String patternToRepeatID) {
		this.patternToRepeatID = patternToRepeatID;
	}

	@Override
	public Statement[] createEsperStatements() {
		//two statements needed per select function
		Statement[] result = new Statement[this.selectFunctions.size() +1];
		
		if (controller != null) {
			//get input event name
			inputEventName = this.controller.getNewEventName(this.patternToRepeatID, this.selectFunctionNumber);			
		}
		
		else if (parser != null) {
			inputEventName = getNewEventNameWithParser(this.patternToRepeatID, this.selectFunctionNumber);
			
		}

		
		/*
		 * build statements
		 */
		String selectClause;
		String fromClause;
		SelFunction sel;
		Statement stat;
		
		//first statement: counting
		selectClause = Constants.EPL_SELECT
					   + " * ";
		fromClause = Constants.EPL_FROM
					 + " "
					 + inputEventName
					 + ".win:length_batch("
					 + this.repetitionCount
					 + ")";
		
		stat = new Statement();
		stat.setSelectFunction(null);
		stat.setStatement(selectClause + fromClause);
		
		//add first statement
		result[0] = stat;

		//further statements: selecting
		fromClause = Constants.EPL_FROM
		 + " "
		 + Constants.EPL_PATTERN
		 + " [every ("
		 + inputEventName
		 + Constants.REPETIVITE_COUNT_EVENT_SUFFIX
		 + " -> "
		 + inputEventName
		 + " = "
		 + inputEventName
		 + ")]";
		
		for (int i = 1; i < result.length; i ++) {
			sel = this.selectFunctions.get(i - 1);
			
			selectClause = Constants.EPL_SELECT
			   + " ";
			
//			if (sel.getFunctionName().equals(Constants.FUNC_SELECT_EVENT_NAME)) {
//				selectClause += inputEventName 
//								+ " as value ";
//			}
//			else {
			selectClause += sel.getSelectString(false);
//			}
			selectClause += " ";
			
			stat = new Statement();
			stat.setSelectFunction(sel);
			stat.setStatement(selectClause + fromClause);
			
			//add statement
			result[i] = stat;
		}
		
		return result;
	}
	
	
	/**
	 * Method for creating statements using just an EMLParser instead 
	 * of EsperController. (added by Matthes)
	 */
	private String getNewEventNameWithParser(String secondPatternID2,
			int selectFunctionNumber) {
		//get all patterns from parser
		HashMap<String, APattern> patterns = parser.getPatterns();
		
		//search for pattern
		if (!this.parser.getPatterns().containsKey(patternID)) {
			return null;
		}
		APattern pattern = patterns.get(patternID);
		
		//search for select function
		if (!(pattern.getSelectFunctions().size() > selectFunctionNumber)) {
			if (!(pattern.getSelectFunctions().size() >= 0)) {
				return null;
			}
			//set number to 0
			selectFunctionNumber = 0;
		}
		
		//return newEventName
		return pattern.getSelectFunctions().get(selectFunctionNumber).getNewEventName();
	}

	@Override
	public Statement[] createEsperStatements(EMLParser parser) {
		this.parser = parser;
		return createEsperStatements();
	}

	/**
	 * @param controller the controller to set
	 */
	public void setController(EsperController controller) {
		this.controller = controller;
	}

	
	/**
	 * sets the select function to use
	 * 
	 * @param selectFunctionNumber number of the select function
	 */
	public void setSelectFunctionToUse(int selectFunctionNumber) {
		this.selectFunctionNumber = selectFunctionNumber;
	}

	/**
	 * @return the inputEventName
	 */
	public String getInputEventName() {
		return this.inputEventName;
	}
}
