/**
 * Part of the diploma thesis of Thomas Everding.
 * @author Thomas Everding
 */

package de.ifgi.lehre.thesisEverding.eml.pattern;

import java.util.Vector;

import de.ifgi.lehre.thesisEverding.eml.Constants;
import de.ifgi.lehre.thesisEverding.eml.esper.EMLParser;
import de.ifgi.lehre.thesisEverding.eml.esper.EsperController;


/**
 * representation of simple patterns
 * 
 * @author Thomas Everding
 * 
 */
public class PatternSimple extends AGuardedViewPattern {
	
	private String inputName;
	
	private Vector<PropRestriction> propertyRestrictions = new Vector<PropRestriction>();
	
	private Statement[] statements = null;
	
	private EsperController controller;
	
	
	/**
	 * 
	 * Constructor
	 * 
	 * @param controller controller of this process
	 */
	public PatternSimple(EsperController controller) {
		this.controller = controller;
	}
	

	/**
	 * @return the inputName
	 */
	public String getInputName() {
		return this.inputName;
	}
	

	/**
	 * @param inputName the inputName to set
	 */
	public void setInputName(String inputName) {
		this.inputName = inputName;
	}
	

	/**
	 * @return the propertyRestrictions
	 */
	public Vector<PropRestriction> getPropertyRestrictions() {
		return this.propertyRestrictions;
	}
	

	/**
	 * adds a property restriction
	 * 
	 * @param restriction the new property restriction
	 */
	public void addPropertyRestriction(PropRestriction restriction) {
		this.propertyRestrictions.add(restriction);
	}
	

	@Override
	public Statement[] createEsperStatements() {
		//lazy load
		if (statements != null) {
			return statements;
		}
		
		/*
		 * statements look like: 
		 * 
		 * select <selectFunction> 
		 * from <inputevents with restrictions> 
		 * where <guard>
		 * 
		 * one statement per select function is built
		 */

		if (this.selectFunctions.size() <= 0) {
			//no select function specified
			String statement = Constants.EPL_SELECT 
							   + " * " 
							   + this.createFromClause() 
							   + " "
							   + this.createWhereClause(false);
			Statement s = new Statement();
			s.setStatement(statement);
			
			SelFunction sel = new SelFunction(this.controller);
			sel.setFunctionName(Constants.FUNC_SELECT_EVENT_NAME);
			sel.setStatement(statement);
			
			s.setSelectFunction(sel);
			
			return new Statement[] { s };
		}
		
		statements = new Statement[this.selectFunctions.size()];
		
		//get from and where clause first, they do not change
		String fromClause = this.createFromClause();
		String whereClause = this.createWhereClause(false);
		
		//get insert and select clause for every select function
		String statement;
		Statement s;
		SelFunction sel;
		for (int i = 0; i < statements.length; i++) {
			sel = this.selectFunctions.get(i);
			
			//			//insert into
			//			statement = this.createInsertClause(sel);
			
			//select
			statement = this.createSelectClause(sel);
			
			//from
			statement += " " + fromClause;
			
			//where
			statement += " " + whereClause;
			
			//add to list
			s = new Statement();
			s.setSelectFunction(sel);
			s.setStatement(statement);
			statements[i] = s;
		}
		
		return statements;
	}
	

	@Override
	public Statement[] createEsperStatements(EMLParser parser) {
		return createEsperStatements();
	}
	
	//	/**
	//	 * creates the insert into clause for a given select function
	//	 * 
	//	 * @param selectFunction the select function
	//	 * 
	//	 * @return the insert into clause
	//	 */
	//	private String createInsertClause(SelFunction selectFunction) {
	//		return Constants.EPL_INSERT
	//			   + " "
	//			   + Constants.EPL_INTO
	//			   + " "
	//			   + selectFunction.getNewEventName();
	//	}
	
	/**
	 * creates the select clause for a select functions
	 * 
	 * @param sel the select function
	 * 
	 * @return the select clause
	 */
	private String createSelectClause(SelFunction sel) {
		String result = Constants.EPL_SELECT + " ";
		
		//		if (sel.getFunctionName().equals(Constants.FUNC_SELECT_EVENT_NAME)) {
		//			//select event on simple patterns always: "select *"
		//			result += "*";
		//		}
		//		else {
		result += sel.getSelectString(true);
		//		}
		return result;
	}
	

	/**
	 * creates the from clause for this pattern
	 * 
	 * @return the from clause
	 */
	private String createFromClause() {
		String clause = Constants.EPL_FROM + " ";
		
		/*
		 * from input
		 */
		clause += this.inputName;
		
		/*
		 * restrictions
		 */
		if (this.propertyRestrictions.size() > 0) {
			boolean first = true;
			clause += "(";
			
			for (PropRestriction res : this.propertyRestrictions) {
				if (!first) {
					clause += ", ";
				}
				
				//add restriction
				clause += res.getName() + "=" + res.getValue();
			}
			
			clause += ")";
		}
		
		/*
		 * view
		 */
		clause += this.view.getViewString();
		
		return clause;
	}
	

	/**
	 * creates the where clause for this pattern
	 * @param fullNames if <code>true</code> the property names are used with the event names, else only the
	 * property names are used
	 * @return the where clause
	 */
	private String createWhereClause(boolean fullNames) {
		if (this.guard != null) {
			//TODO hier nur lvl. 2
			String pName = null;
			if (selectFunctions.size() != 0) {
				Object o = this.selectFunctions.get(0).getFunctionParameters().get(Constants.FUNC_SELECT_PROPERTY_NAME); 
				if (o != null) {
					pName = o.toString();
				}
			}
			return this.guard.createStatement(fullNames, pName);
		}
		return "";
	}
}