/**
 * Part of the diploma thesis of Thomas Everding.
 * @author Thomas Everding
 */

package de.ifgi.lehre.thesisEverding.eml.pattern;

import java.util.HashMap;
import java.util.logging.Logger;

import de.ifgi.lehre.thesisEverding.eml.Constants;
import de.ifgi.lehre.thesisEverding.eml.esper.EsperController;
import de.ifgi.lehre.thesisEverding.eml.event.MapEvent;


/**
 * represents a single select function
 * 
 * @author Thomas Everding
 * 
 */
public class SelFunction {
	
	/*
	 * Logger instance for this class
	 */
	private Logger logger = Logger.getLogger(this.getClass().getName());
	
	private String outputName = "";
	
	private boolean createCausality = false;
	
	private String selectString = "";
	
	private String newEventName = "";
	
	private String functionName;
	
	private HashMap<String, Object> functionParameters;
	
	private HashMap<String, String> fullPropertyNames;
	
	private String statement;
	
	private boolean singleValueOutput;
	
	private EsperController controller;
	
	/**
	 * contains the data types of the resulting event or the inner (nested) types, if the resulting event contains
	 * an event.
	 */
	@SuppressWarnings("unchecked")
	private HashMap<String, Object> dataTypes;
	
	
	/**
	 * 
	 * Constructor
	 * 
	 * @param controller controller of this process
	 * 
	 */
	@SuppressWarnings("unchecked")
	public SelFunction(EsperController controller) {
		this.functionParameters = new HashMap<String, Object>();
		this.fullPropertyNames = new HashMap<String, String>();
		this.dataTypes = new HashMap<String, Object>();
		
		this.controller = controller;
	}
	

	/**
	 * @return the outputName
	 */
	public String getOutputName() {
		return this.outputName;
	}
	

	/**
	 * @param outputName the outputName to set
	 */
	public void setOutputName(String outputName) {
		this.outputName = outputName;
	}
	

	/**
	 * @return the createCausality
	 */
	public boolean getCreateCausality() {
		return this.createCausality;
	}
	

	/**
	 * @param createCausality the createCausality to set
	 */
	public void setCreateCausality(boolean createCausality) {
		this.createCausality = createCausality;
	}
	

	/**
	 * @return the newEventName
	 */
	public String getNewEventName() {
		return this.newEventName;
	}
	

	/**
	 * @param newEventName the newEventName to set
	 */
	public void setNewEventName(String newEventName) {
		this.newEventName = newEventName;
	}
	

	/**
	 * @return the functionName
	 */
	public String getFunctionName() {
		return this.functionName;
	}
	

	/**
	 * @param functionName the functionName to set
	 */
	public void setFunctionName(String functionName) {
		this.functionName = functionName;
	}
	

	/**
	 * @return the functionParameters
	 */
	public HashMap<String, Object> getFunctionParameters() {
		return this.functionParameters;
	}
	

	/**
	 * adds a single parameter to the function parameter map
	 * 
	 * @param pName name of the parameter
	 * @param pValue value of the parameter
	 */
	public void addFunctionParameter(String pName, Object pValue) {
		if (pName.equals(Constants.SELECT_PARAM_PROPERTY_NAME)) {
			//parse property name
			pValue = this.parsePropertyName(pValue.toString());
		}
		this.functionParameters.put(pName, pValue);
	}
	

	/**
	 * parses a property name for use in esper
	 * 
	 * @param name the property name in EML
	 */
	private String parsePropertyName(String name) {
		String result = name;
		//		int i = name.indexOf("/");
		//		
		//		if (i >= 0) {
		//			//cut off the event name
		//			result = name.substring(i + 1);
		//		}
		//		else {
		//			result = name;
		//		}
		
		//replace "/" by "."
		result = result.replaceAll("/", ".");
		
		//save property name
		this.fullPropertyNames.put(result, name);
		
		return result;
	}
	

	/**
	 * 
	 * @param forSimplePattern set to <code>true</code> if called for a simple pattern
	 * @return the select string, creates it if necessary
	 */
	public String getSelectString(boolean forSimplePattern) {
		if (this.selectString.equals("")) {
			//create select string
			
			/*
			 * SelectEvent
			 */
			if (functionName.equals(Constants.FUNC_SELECT_EVENT_NAME)) {
				this.singleValueOutput = false;
				
				if (this.functionParameters.containsKey(Constants.SELECT_PARAM_EVENT_NAME)) {
					//with EventName given
					String eventName = "" + this.functionParameters.get(Constants.SELECT_PARAM_EVENT_NAME);
					if (forSimplePattern) {
						//named not possible in simple patterns
						this.selectString = "*";
					}
					else {
						//select specified event
						this.selectString = "" + eventName + " as value";
					}
					
					//set data type
					this.dataTypes.put(MapEvent.VALUE_KEY, this.controller.getEventDatatype(eventName));
				}
				else {
					logger.severe("No event name given.");
					//					//no EventName given
					//					this.selectString = "*";
					//					
					//					//set data type
					//					this.dataTypes.put(MapEvent.VALUE_KEY, MapEvent.class);
				}
			}
			
			/*
			 * SelectProperty
			 */
			else if (functionName.equals(Constants.FUNC_SELECT_PROPERTY_NAME)) {
				this.singleValueOutput = true;
				
				//load property data type
				String fullPropertyName = this.fullPropertyNames.get(this.functionParameters.get(Constants.SELECT_PARAM_PROPERTY_NAME));
				this.dataTypes.put(MapEvent.VALUE_KEY, this.controller.getDatatype(fullPropertyName));
				
				//adjust property name for simple patterns
				String pName = this.functionParameters.get(Constants.SELECT_PARAM_PROPERTY_NAME).toString();
				if (forSimplePattern) {
					if (pName.contains(".")) {
						//exclude event name
						pName = pName.substring(pName.indexOf(".") + 1);
					}
				}
				//pName.replaceAll(":", "__");
				
				//build select string
				this.selectString = "" + pName + " as value";
			}
			
			/*
			 * SelectSum
			 */
			else if (functionName.equals(Constants.FUNC_SELECT_SUM_NAME)) {
				this.singleValueOutput = true;
				this.dataTypes.put(MapEvent.VALUE_KEY, Number.class);
				
				//adjust property name for simple patterns
				String pName = this.functionParameters.get(Constants.SELECT_PARAM_PROPERTY_NAME).toString();
				if (forSimplePattern) {
					if (pName.contains(".")) {
						//exclude event name
						pName = pName.substring(pName.indexOf(".") + 1);
					}
				}
				//pName.replaceAll(":", "__");
				
				this.selectString = "sum(" + pName + ")" + " as value";
			}
			
			/*
			 * SelectAvg
			 */
			else if (functionName.equals(Constants.FUNC_SELECT_AVG_NAME)) {
				this.singleValueOutput = true;
				this.dataTypes.put(MapEvent.VALUE_KEY, Number.class);
				
				//adjust property name for simple patterns
				String pName = this.functionParameters.get(Constants.SELECT_PARAM_PROPERTY_NAME).toString();
				if (forSimplePattern) {
					if (pName.contains(".")) {
						//exclude event name
						pName = pName.substring(pName.indexOf(".") + 1);
					}
				}
				//pName.replaceAll(":", "__");
				
				this.selectString = "avg(" + pName + ")" + " as value";
			}
			
			/*
			 * SelectMax
			 */
			else if (functionName.equals(Constants.FUNC_SELECT_MAX_NAME)) {
				this.singleValueOutput = true;
				this.dataTypes.put(MapEvent.VALUE_KEY, Number.class);
				
				//adjust property name for simple patterns
				String pName = this.functionParameters.get(Constants.SELECT_PARAM_PROPERTY_NAME).toString();
				if (forSimplePattern) {
					if (pName.contains(".")) {
						//exclude event name
						pName = pName.substring(pName.indexOf(".") + 1);
					}
				}
				//pName.replaceAll(":", "__");
				
				this.selectString = "max(" + pName + ")" + " as value";
			}
			
			/*
			 * SelectMin
			 */
			else if (functionName.equals(Constants.FUNC_SELECT_MIN_NAME)) {
				this.singleValueOutput = true;
				this.dataTypes.put(MapEvent.VALUE_KEY, Number.class);
				
				//adjust property name for simple patterns
				String pName = this.functionParameters.get(Constants.SELECT_PARAM_PROPERTY_NAME).toString();
				if (forSimplePattern) {
					if (pName.contains(".")) {
						//exclude event name
						pName = pName.substring(pName.indexOf(".") + 1);
					}
				}
				//pName.replaceAll(":", "__");
				
				this.selectString = "min(" + pName + ")" + " as value";
			}
			
			/*
			 * SelectCount
			 */
			else if (functionName.equals(Constants.FUNC_SELECT_COUNT_NAME)) {
				this.singleValueOutput = true;
				this.dataTypes.put(MapEvent.VALUE_KEY, Integer.class);
				this.selectString = "count(*) as value";
			}
			
			/*
			 * NotifyOnSelect
			 */
			else if (functionName.equals(Constants.FUNC_NOTIFY_ON_SELECT_NAME)) {
				this.singleValueOutput = true;
				this.dataTypes.put(MapEvent.VALUE_KEY, String.class);
				this.selectString = "\"" + this.functionParameters.get(Constants.SELECT_PARAM_MESSAGE_NAME)
						+ "\" as value";
			}
			
			/*
			 * SelectStdDev (user defined)
			 */
			else if (functionName.equals(Constants.FUNC_SELECT_STDDEV_NAME)) {
				this.singleValueOutput = true;
				this.dataTypes.put(MapEvent.VALUE_KEY, Number.class);
				
				//adjust property name for simple patterns
				String pName = this.functionParameters.get(Constants.SELECT_PARAM_PROPERTY_NAME).toString();
				if (forSimplePattern) {
					if (pName.contains(".")) {
						//exclude event name
						pName = pName.substring(pName.indexOf(".") + 1);
					}
				}
				//pName.replaceAll(":", "__");
				
				this.selectString = "stddev(" + pName + ")" + " as value";
			}
			
			/*
			 * parse additional select functions here
			 */
		}
		return this.selectString;
	}
	

	/**
	 * @return the statement
	 */
	public String getStatement() {
		return this.statement;
	}
	

	/**
	 * @param statement the statement to set
	 */
	public void setStatement(String statement) {
		this.statement = statement;
	}
	

	/**
	 * @return the singleValueOutput
	 */
	public boolean isSingleValueOutput() {
		return this.singleValueOutput;
	}
	

	/**
	 * @return the dataTypes
	 */
	@SuppressWarnings("unchecked")
	public HashMap<String, Object> getDataTypes() {
		return this.dataTypes;
	}
}
