/**
 * Part of the diploma thesis of Thomas Everding.
 * @author Thomas Everding
 */

package de.ifgi.lehre.thesisEverding.process;

import javax.swing.JDialog;

/**
 * Abstract representation of a single process
 * 
 * @author Thomas Everding
 *
 */
public abstract class AProcess {
	
	protected InputDescription[]  inputDescriptions;
	protected InputDescription[]  parameterDescriptions;
	protected OutputDescription[] outputDescriptions;
	protected AProcessExecutor executor;
	protected JDialog configGUI;
	
	
	/**
	 * @return the executor for this process plugin
	 */
	public AProcessExecutor getExecutor() {
		return executor;
	}
	
	
	/**
	 * shows the GUI to configure
	 * the process plugin
	 */
	public void showConfigGUI() {
		configGUI.setModal(true);
		configGUI.setVisible(true);
	}
	
	
	/**
	 * indicates if the process plugin is ready to use
	 * @return 'true' if the process is ready to use, 'false' if not
	 */
	public abstract boolean isReady();
	
	
	/**
	 * 
	 * @return the name of the process
	 */
	public abstract String getName();
	
	
	/**
	 * sets the name of a process
	 * 
	 * @param newName the new name
	 */
	public abstract void setName(String newName);


	/**
	 * @return an array of {@link InputDescription}s
	 * @see InputDescription
	 */
	public InputDescription[] getInputDescriptions() {
		return this.inputDescriptions;
	}
	
	
	/**
	 * @return an array of {@link InputDescription}s
	 * @see InputDescription
	 */
	public InputDescription[] getParameterDescriptions() {
		return this.parameterDescriptions;
	}


	/**
	 * @return an array of {@link OutputDescription}s
	 * @see OutputDescription
	 */
	public OutputDescription[] getOutputDescriptions() {
		return this.outputDescriptions;
	}
	
	
//	/**
//	 * Configures the process.
//	 * 
//	 * @param parameters vector with all needed parameters.
//	 */
//	public abstract void configure(Vector<Object> parameters);
//	
}
