/**
 * Part of the diploma thesis of Thomas Everding.
 * @author Thomas Everding
 */

package de.ifgi.lehre.thesisEverding.processCommunication;

import de.ifgi.lehre.thesisEverding.util.DataBuffer;

/**
 * listens on a data stream an pushes the data to a {@link DataBuffer}
 * 
 * @author Thomas Everding
 *
 */
public class BufferedListener implements Runnable{
	
	private boolean running = true;
	
	private DataBuffer buffer;
	
	private String inputName;
	
	private IInputConnector in;
	
	
	/**
	 * 
	 * Constructor
	 * 
	 * @param in the input connector this is listening on
	 * @param buffer the {@link DataBuffer} for the data
	 * @param inputName the name of the inputs this is listening for
	 */
	public BufferedListener (IInputConnector in, DataBuffer buffer, String inputName) {
		this.in = in;
		this.buffer = buffer;
		this.inputName = inputName;
	}

	public void run() {
		
		if (in == null) {
			//no input
			return;
		}
		
		if (buffer == null) {
			//no output
			return;
		}
		
		Object obj;
		while (running) {
			//read data
			obj = in.read();
			
			//write data
			buffer.setData(inputName, obj);
			
			//yield
			Thread.yield();
		}
	}

	/**
	 * @param running set to <code>false</code> to stop the listener
	 */
	public void setRunning(boolean running) {
		this.running = running;
	}
}
