/**
 * Part of the diploma thesis of Thomas Everding.
 * @author Thomas Everding
 */

package de.ifgi.lehre.thesisEverding.processCommunication;


/**
 * listens on a data stream an pushes the data to a {@link IInputReceiver}
 * 
 * @author Thomas Everding
 *
 */
public class NonBufferedListener implements Runnable{
	
	private IInputReceiver receiver;
	
	private IInputConnector connector;
	
	private String inputName;
	
	private boolean running;
	
	
	/**
	 * 
	 * Constructor
	 *
	 * @param inputName Name of the input
	 * @param connector Connection to the input source
	 * @param receiver Receiver for the input
	 */
	public NonBufferedListener(String inputName, IInputConnector connector, IInputReceiver receiver) {
		this.connector = connector;
		this.receiver = receiver;
		this.inputName = inputName;
		
		this.running = true;
	}

	public void run() {
		//check listener state
		if (connector == null) {
			return;
		}
		
		if (receiver == null) {
			return;
		}
		
		//listen for inputs
		Object value;
		while(running) {
			//read data
			value = this.connector.read();
			
			//write data
			receiver.receiveInput(this.inputName, value);
			
			//yield
			Thread.yield();
		}
	}

	/**
	 * Stops the listener
	 * 
	 * @param running set to <code>false</code> to stop the listener
	 */
	public void setRunning(boolean running) {
		this.running = running;
	}
	
	
	
}
