/**
 * Part of the diploma thesis of Thomas Everding.
 * @author Thomas Everding
 */

package de.ifgi.lehre.thesisEverding.util;

import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;

/**
 * ThreadPool for the execution of {@link Runnable}s.
 * Implemented as Singleton. 
 * 
 * @author Thomas Everding
 *
 */
public class ThreadPool {
	
	private static ThreadPool instance = null;
	
	private ExecutorService executor;
	
	/**
	 * 
	 * Private Constructor
	 *
	 */
	private ThreadPool() {
		executor = Executors.newCachedThreadPool();
	}
	
	
	/**
	 * 
	 * @return the only instance of this class
	 */
	public static synchronized ThreadPool getInstance() {
		if (instance == null) {
			instance = new ThreadPool();
		}
		
		return instance;
	}
	
	
	/**
	 * Executes a class implementing {@link Runnable}.
	 * Does not block.
	 * 
	 * @param runnable the runnable
	 */
	@SuppressWarnings("unchecked")
	public synchronized void execute(Runnable runnable) {
		executor.submit(runnable);
	}
}
