/***************************************************************
Copyright 2007 52North Initiative for Geospatial Open Source Software GmbH

 Author: jtheuer, University of Muenster

 Contact: Andreas Wytzisk, 
 52North Initiative for Geospatial Open Source SoftwareGmbH, 
 Martin-Luther-King-Weg 24,
 48155 Muenster, Germany, 
 info@52north.org

 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
 version 2 as published by the Free Software Foundation.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; even without the implied WARRANTY OF
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program (see gnu-gpl v2.txt). If not, write to
 the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 Boston, MA 02111-1307, USA or visit the Free
 Software Foundations web page, http://www.fsf.org.

 ***************************************************************/

	
package org.n52.swe.sas.muse.common;
import java.lang.reflect.Method;
import java.util.logging.*;

import javax.xml.namespace.QName;

import org.apache.muse.core.routing.AbstractMessageHandler;
import org.apache.muse.util.ReflectUtils;
import org.apache.muse.ws.addressing.soap.SoapFault;
import org.apache.muse.ws.resource.impl.AbstractWsResourceCapability;
import org.w3c.dom.Element;

/**
 * @author Jan Torben Heuer <jan.heuer@uni-muenster.de>
 * 
 * Simple Handler that just passes through
 *
 */
public class SimpleHandler extends AbstractMessageHandler {
	/* automatically generated Logger */@SuppressWarnings("unused")
	private static final Logger LOGGER = Logger.getLogger(SimpleHandler.class.getName());

	/**
	 * Creates a new Handler that just passes every request or response to the next layer
	 * @param actionURI the URI of the associates action
	 * @param requestQName the QName
	 * @param methodName the name of the method that should be called
	 */
	public SimpleHandler(String actionURI, QName requestQName, AbstractWsResourceCapability capability, String methodName) {
		super(actionURI, requestQName);
		
		Method method = ReflectUtils.getFirstMethod(capability.getClass(), methodName);
		this.setMethod(method);

	}

	/* (non-Javadoc)
	 * @see org.apache.muse.core.routing.MessageHandler#fromXML(org.w3c.dom.Element)
	 */
	public Object[] fromXML(Element xml) throws SoapFault {
		return new Object[]{xml};
	}

	/* (non-Javadoc)
	 * @see org.apache.muse.core.routing.MessageHandler#toXML(java.lang.Object)
	 */
	public Element toXML(Object result) throws SoapFault {
		return (Element) result;
	}
}
