package org.n52.swe.sas.muse.filter;

import java.util.*;

import org.apache.muse.util.xml.XmlUtils;
import org.apache.muse.ws.notification.*;
import org.w3c.dom.Document;
import org.w3c.dom.Element;

/**
 * 
 * @author Matthes Rieke <m.rieke@uni-muenster.de>
 *
 */
public class SESFilterCollection implements Filter {

	private Collection<Filter> _filters = new ArrayList<Filter>();
	
	
	/**
	 * Holds the SESConstraintFilter (can only appear once
	 * per SESFilterCollection?!) 
	 */
	private SESConstraintFilter constraintFilter = null;

	public boolean accepts(NotificationMessage message) {
		Iterator<Filter> i = getFilters().iterator();

		while (i.hasNext())	{
			Filter next = i.next();

			//
			// only one filter has to fail for the whole thing to fail.
			// only check non SESConstraintFilter
			//
			if ( !(next instanceof SESConstraintFilter)){
				if (!next.accepts(message))
					return false;	
			}
			
		}

		return true;
	}

	public void addFilter(Filter filter) {
		if (filter instanceof SESConstraintFilter && constraintFilter == null) {
			constraintFilter = (SESConstraintFilter) filter;
		} else if (filter instanceof SESConstraintFilter && constraintFilter != null) {
			//TODO log error
			Throwable e = new Throwable("Only one constraint filter per subscription is allowed.");
			e.printStackTrace();
		}
		_filters.add(filter);
	}

	public Collection<Filter> getFilters() {
		return Collections.unmodifiableCollection(_filters);
	}

	/**
	 * Holds the SESConstraintFilter (can only appear once
	 * per SESFilterCollection?!) 
	 */
	public SESConstraintFilter getConstraintFilter() {
		return constraintFilter;
	}

	public Element toXML() {
		return toXML(XmlUtils.EMPTY_DOC);
	}

	public Element toXML(Document doc) {
		Element filterXML = XmlUtils.createElement(doc, WsnConstants.FILTER_QNAME);

		Iterator<Filter> i = getFilters().iterator();

		while (i.hasNext())
		{
			Filter next = i.next();
			Element nextXML = next.toXML(doc);

			//
			// we have to 'move' instead of 'appendChild' because the other 
			// Filter types already add a <Filter/> element as part of their 
			// toXML() implementations, and we can't change this for reasons 
			// of backwards compatibility. therefore, we just take the element 
			// under the <Filter/> and move it under our new <Filter/>
			//
			XmlUtils.moveSubTree(nextXML, filterXML);
		}

		return filterXML;
	}

}