/***************************************************************
Copyright 2007 52North Initiative for Geospatial Open Source Software GmbH

 Author: jtheuer, University of Muenster

 Contact: Andreas Wytzisk, 
 52North Initiative for Geospatial Open Source SoftwareGmbH, 
 Martin-Luther-King-Weg 24,
 48155 Muenster, Germany, 
 info@52north.org

 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
 version 2 as published by the Free Software Foundation.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; even without the implied WARRANTY OF
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program (see gnu-gpl v2.txt). If not, write to
 the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 Boston, MA 02111-1307, USA or visit the Free
 Software Foundations web page, http://www.fsf.org.

 ***************************************************************/

package org.n52.swe.sas.muse.producer.dummy;

import java.io.IOException;
import java.util.logging.Level;
import java.util.logging.Logger;

import javax.xml.namespace.QName;

import org.apache.muse.ws.notification.NotificationProducer;
import org.apache.xmlbeans.XmlException;
import org.w3c.dom.Element;

/**
 * @author Jan Torben Heuer <jan.heuer@uni-muenster.de>
 *
 */
public abstract class AbstractProducer {
	/* automatically generated Logger */@SuppressWarnings("unused")
	static final Logger LOGGER = Logger.getLogger(AbstractProducer.class.getName());


	protected int period;
	protected NotificationProducer producer;
	protected QName topic;

	/**
	 * @param period how long the thread will sleep (in ms) between two publish calls
	 * @param producer the instance where {@link NotificationProducer#publish(QName, Element)} is called
	 * @param topic a topic that is supplied with the o&m message
	 * @throws XmlException
	 * @throws IOException
	 */
	public AbstractProducer(int period, NotificationProducer producer, QName topic) {
		this.producer=producer;
		this.period = period;
		this.topic = topic;
	}

	protected abstract void sendAlert();
	
	class ProducerThread extends Thread {

		@Override
		public void run() {
			while(true) {
				
				sendAlert();
				
				try {
					sleep(period);
				} catch (InterruptedException e) {
					LOGGER.log(Level.SEVERE,"autogenerated catch-block", e);
				}
			}
		}
		
	}

}