package org.n52.swe.sas.muse.unitconversion;

/**
 * 
 * @author Matthes Rieke <m.rieke@uni-muenster.de>
 *
 */
public interface IUnitConverter {
	
	/**
	 * Converts the given value to its base units and returns the new
	 * ucum-expression (result[0]) and the converted value (result[1]).
	 * Should be called for each subscription.
	 * 
	 */
	public abstract Object[] convert(String unitString, double value);
	
	/**
	 * Used to convert a value from the given unit to the first
	 * registered compatbile unit, using UCUM. returns the new
	 * ucum-expression (result[0]) and the converted value (result[1]).
	 * Should be called for every incoming sensor data.
	 * 
	 * @throws Exception if the formerly registered unit for the
	 * phenomenon is not compatible with the given unit.
	 */
	public abstract Object[] convert(String unitString, String phenomenon, double value) throws Exception;
	
	/**
	 * Used to register a new Unit with a phenomenon. The unit
	 * is converted to its base units (SI).
	 * Should be called for each PublisherRegistration (new sensor) request.
	 * 
	 * @return false If the phenomenon is already registered and is compatible.
	 * @throws Exception If a phenomenon is already registered
	 * but the registered unit is not compatible. 
	 */
	public abstract boolean registerNewUnit(String unitString, String phenomenon) throws Exception;

	/**
	 * Used to check if the phenomenon is compatible with a possibly registered one.
	 * Should be called for each subscription.
	 * 
	 * @return true if units are compatible or phenomenon was not registered previously
	 * (subscribing should continue normally). false if units are not compatible.
	 */
	public abstract boolean isCompatibleWithPhenomenon(String unitString, String phenomenon);
}
