package org.n52.swe.sas.muse.unitconversion;


import java.util.HashMap;
import java.util.Map;
import java.util.logging.Logger;

import org.vast.unit.*;

/**
 * 
 * @author Matthes Rieke <m.rieke@uni-muenster.de>
 *
 */
public class SESUnitConverter implements IUnitConverter {
	

	private UnitParser parser;
	private Map<String, Unit> unitPhenomenonMap;
	private Logger logger;


	public SESUnitConverter(Logger log) {
		logger = log;
		parser = new UnitParserUCUM();
		unitPhenomenonMap = new HashMap<String, Unit>();
	}

	public Object[] convert(String unitString, double value) {
		Object[] result = new Object[2];
		result[0] = parser.getUnit(unitString).getCompatibleSIUnit();
		result[1] = UnitConversion.getConverter(parser.getUnit(unitString),
				(Unit) result[0]).convert(value);
		result[0] = ((Unit) result[0]).getUCUMExpression();
		return result;
	}
	
	public Object[] convert(String unitString, String phenomenon, double value) throws Exception {
		Unit unit = parser.getUnit(unitString);
		
		Unit unit2 = unitPhenomenonMap.get(phenomenon);
		if (unit2 == null) {
			getLog().warning("Phenomenon not registered. Should not happen, sensor should have" +
					"registered phenomenon. Using base units for conversion.");
			unit2 = unit.getCompatibleSIUnit();
		}
		
		if (!unit.isCompatible(unit2)) {
			throw new Exception("The registered SI unit for this phenomenon is not" +
					"compatible with the given unit.");
		}
		Object[] result = new Object[2];
		result[0] = unit;
		result[1] = UnitConversion.getConverter(unit,
				unit2).convert(value);
		//TODO check if conversion was possible
		return result; 
	}


	private Logger getLog() {
		return logger;
	}

	public boolean registerNewUnit(String unitString, String phenomenon) throws Exception {
		Unit unit = parser.getUnit(unitString).getCompatibleSIUnit();
		if (unitPhenomenonMap.containsKey(phenomenon)) {
			if (!unitPhenomenonMap.get(phenomenon).isCompatible(unit)) {
				throw new Exception("Phenomenon already registered, but registered" +
						" units are not compatible!");
			}
			//return false: phenomnen already registered with comparable unit
			return false;
		}
		unitPhenomenonMap.put(phenomenon, unit);
		//return true: new phenomenon registered
		return true;
	}

	public boolean isCompatibleWithPhenomenon(String unitString, String phenomenon) {
		Unit unit = parser.getUnit(unitString).getCompatibleSIUnit();
		Unit unit2 = unitPhenomenonMap.get(phenomenon);
		if (unit2 != null) {
			return unit2.isCompatible(unit);
		}
		//return true: no phenomenon registered - continue
		return true;
	}



}
