/***************************************************************
Copyright 2007 52North Initiative for Geospatial Open Source Software GmbH

 Author: jtheuer, University of Muenster

 Contact: Andreas Wytzisk, 
 52North Initiative for Geospatial Open Source SoftwareGmbH, 
 Martin-Luther-King-Weg 24,
 48155 Muenster, Germany, 
 info@52north.org

 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
 version 2 as published by the Free Software Foundation.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; even without the implied WARRANTY OF
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program (see gnu-gpl v2.txt). If not, write to
 the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 Boston, MA 02111-1307, USA or visit the Free
 Software Foundations web page, http://www.fsf.org.

 ***************************************************************/

	
package org.n52.swe.sas.muse.util;
import java.util.HashMap;
import java.util.Map;
import java.util.logging.Logger;

import org.apache.muse.core.Resource;
import org.apache.muse.core.ResourceManagerListener;
import org.apache.muse.ws.addressing.EndpointReference;
import org.apache.muse.ws.addressing.soap.SoapFault;

/**
 * @author Jan Torben Heuer <jan.heuer@uni-muenster.de>
 *
 */
public class FilterListener<T> implements ResourceManagerListener {
	/* automatically generated Logger */@SuppressWarnings("unused")
	private static final Logger LOGGER = Logger.getLogger(FilterListener.class.getName());

	private Class<T> clazz;
	private AdvancedResourceManagerListener<T> listener;
	private Map<EndpointReference, T> map = new HashMap<EndpointReference, T>();
	
	/**
	 * Creates filter for a {@link ResourceManagerListener}. Only Resources from the given type are forwared to the listener
	 * @param clazz
	 */
	public FilterListener(Class<T> clazz, AdvancedResourceManagerListener<T> listener) {
		this.clazz = clazz;
		this.listener = listener;
	}

	/* (non-Javadoc)
	 * @see org.apache.muse.core.ResourceManagerListener#resourceAdded(org.apache.muse.ws.addressing.EndpointReference, org.apache.muse.core.Resource)
	 */
	@SuppressWarnings("unchecked")
	public void resourceAdded(EndpointReference epr, Resource resource) throws SoapFault {
		if(resource.getClass().equals(clazz)) {
			map.put(epr, (T) resource);
			listener.resourceAdded(epr, (T) resource);
		}
	}

	/* (non-Javadoc)
	 * @see org.apache.muse.core.ResourceManagerListener#resourceRemoved(org.apache.muse.ws.addressing.EndpointReference)
	 */
	public void resourceRemoved(EndpointReference epr) throws SoapFault {
		T resource = map.get(epr);
		if(resource != null) {
			listener.resourceRemoved(epr, resource);
		}
	}

}
