// 
// MyCapability.java
// Wed May 21 14:00:09 CEST 2008
// Generated by the Apache Muse Code Generation Tool
// 
package org.n52.swe.sas.muse.wsbr;

import java.net.URI;
import java.net.URISyntaxException;
import java.util.Date;
import java.util.Iterator;
import java.util.logging.Level;
import java.util.logging.Logger;

import javax.xml.namespace.QName;

import org.apache.muse.core.Resource;
import org.apache.muse.util.xml.XmlUtils;
import org.apache.muse.ws.addressing.EndpointReference;
import org.apache.muse.ws.addressing.soap.SoapFault;
import org.apache.muse.ws.notification.NotificationProducer;
import org.apache.muse.ws.resource.impl.AbstractWsResourceCapability;
import org.apache.muse.ws.resource.lifetime.ScheduledTermination;
import org.apache.muse.ws.resource.lifetime.faults.ResourceNotDestroyedFault;
import org.n52.swe.sas.muse.wsn.SESSubscriptionManager;
import org.w3c.dom.*;

/**
 * @author Jan Torben Heuer <jan.heuer@uni-muenster.de>
 * 
 * The PRM capability
 */
public class PublisherEndpoint extends AbstractWsResourceCapability implements IPublisherEndpoint, IDestroyRegistration {
	/* automatically generated Logger */@SuppressWarnings("unused")
	private static final Logger LOGGER = Logger.getLogger(PublisherEndpoint.class.getName());

	private Element sensorML;
	private QName[] topics;
	private static String demand = "foo";

	QName[] PROPERTIES = new QName[] { WsbrConstants.CREATION_QNAME, WsbrConstants.PUBLISHER_REFERENCE_QNAME, WsbrConstants.DEMAND, WsbrConstants.TOPIC };

	// WsbrConstants.CREATION_QNAME, , ,
	// ,
	public Element renewRegistration(Element terminationTime) throws Exception {
		Date date = XmlUtils.getDate(terminationTime);

		ScheduledTermination scheduledTermination = (ScheduledTermination) getResource().getCapability(
				"http://docs.oasis-open.org/wsrf/rlw-2/ScheduledResourceTermination");
		scheduledTermination.setTerminationTime(date);

		Element root = XmlUtils.createElement(SesConstants.RENEW_REGISTRATION_RESPONSE);
		// TODO: use joda time (?)
		XmlUtils.setElementText(root, date.toLocaleString());
		return root;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.n52.swe.sas.muse.wsrp.IDestroyRegistration#destroyRegistration(org.w3c.dom.Element)
	 * 
	 */
	public Element destroyRegistration(Element something) throws ResourceNotDestroyedFault {
		// see: SimpleImmediateTermination
		try {
			//
			// all we're doing is making the shutdown task public...
			//
			getResource().shutdown();
		}

		catch (SoapFault fault) {
			throw new ResourceNotDestroyedFault(fault);
		}

		Element root = XmlUtils.createElement(SesConstants.DESTROY_REGISTRATION_RESPONSE);
		return root;
	}

	@Override
	public QName[] getPropertyNames() {
		return PROPERTIES;
	}

	@Override
	public void initialize() throws SoapFault {
		super.initialize();

	}

	/**
	 * looks up the <em>first</em> {@link NotificationProducer} instance
	 * 
	 * @return
	 * @throws SoapFault
	 */
	@SuppressWarnings("unchecked")
	private Resource getNotificationProducer() throws SoapFault {
		Resource wsn = null;
		try {
			Iterator<EndpointReference> it = getResource().getResourceManager().getResourceEPRs();
			EndpointReference ref;
			while(it.hasNext()) {
				ref = it.next();
				if(ref.getAddress().toString().endsWith(SESSubscriptionManager.CONTEXT_PATH)) {
					wsn = getResource().getResourceManager().getResource(ref);
					break;
				}
			}
			
			if (wsn == null) {
				throw new SoapFault("Cannot find the NotificationProducer '" + SESSubscriptionManager.CONTEXT_PATH + "'!");
			}
		} catch (RuntimeException e) {
			throw new SoapFault("Invalid NotificationProducer returned: " + wsn != null ? wsn.getClass().getSimpleName() : "", e);
		}
		return wsn;

	}

	@Override
	public void initializeCompleted() throws SoapFault {
		super.initializeCompleted();
	}

	/**
	 * Capability creation time
	 * 
	 * @return
	 */
	public Date getCreationTime() {
		return new Date();
	}

	/**
	 * @return the notification producer then Publisher must use to send events
	 */
	public EndpointReference getPublisherReference() {
		return getWsResource().getEndpointReference();
	}

	/**
	 * @return the notification producer then Publisher must use to send events
	 */
	public EndpointReference getNotificationProducerReference() {

		EndpointReference publisherEpr = null;
		try {
			publisherEpr = getNotificationProducer().getEndpointReference();
		} catch (Exception e) {
			LOGGER.log(Level.WARNING, "Cannot find the NotificationProducer", e);
		}

		return publisherEpr;

	}

	/**
	 * Capability: demand
	 * 
	 * @return
	 */
	public Object getDemand() {
		// TODO: implement
		return demand;
	}

	/**
	 * Capability: SensorML
	 * 
	 * @return
	 */
	public Element getSensorML() {
		return sensorML;
	}

	/**
	 * Sets the sensorML document. Warning, there is no validation!
	 * 
	 * @param sensorML
	 */
	public void setSensorML(Element sensorML) {
		this.sensorML = sensorML;
	}

	/**
	 * The unique sensor-id from the sensorML
	 * 
	 * @return the id as string
	 */
	public String getSensorId() throws SoapFault {
		/* traverse xml tree */
		try {
			if (sensorML == null) {
				LOGGER.info("SensorML is null");
				throw new SoapFault("SensorML document hasn't been supplied for this sensor!");
			}
			// Element system = XmlUtils.findFirstInSubTree(sensorML,
			// SensorMLConstants.SYSTEM);
			Element identification = XmlUtils.findFirstInSubTree(sensorML, SensorMLConstants.IDENTIFICATION);

			Element term = XmlUtils.findFirstInSubTree(identification, SensorMLConstants.TERM);
			if (SensorMLConstants.SENSOR_ID_ATTRIBUTE.equals(term.getAttribute("definition"))) {
				Element value = XmlUtils.getElement(term, SensorMLConstants.VALUE);
				Node text = value.getFirstChild();
				if(text instanceof Text) {
					return ((Text) text).getData();
				}
			}
		} catch (DOMException e) {
			LOGGER.info(e.getMessage());
		} catch (NullPointerException e) {
			LOGGER.info(e.getMessage());
		} catch (Exception e) {
			LOGGER.log(Level.WARNING, "Unrecoverable error: ", e);
		}

		return "invalid-id";
	}

	/**
	 * Capability: Topic
	 * 
	 * @return
	 */
	public QName[] getTopic() {
		return topics;
	}

	/**
	 * Sets this resource's topic. Used by the {@link RegisterPublisher} class.
	 * 
	 * @param topic
	 */
	public void setTopic(QName[] topic) {
		this.topics = topic;
	}
}