/***************************************************************
Copyright 2007 52North Initiative for Geospatial Open Source Software GmbH

 Author: jtheuer, University of Muenster

 Contact: Andreas Wytzisk, 
 52North Initiative for Geospatial Open Source SoftwareGmbH, 
 Martin-Luther-King-Weg 24,
 48155 Muenster, Germany, 
 info@52north.org

 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
 version 2 as published by the Free Software Foundation.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; even without the implied WARRANTY OF
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program (see gnu-gpl v2.txt). If not, write to
 the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 Boston, MA 02111-1307, USA or visit the Free
 Software Foundations web page, http://www.fsf.org.

 ***************************************************************/

package org.n52.swe.sas.muse.wsbr.soaphandler;

import java.util.Date;
import java.util.logging.Logger;

import org.apache.muse.core.routing.AbstractMessageHandler;
import org.apache.muse.ws.addressing.soap.SoapFault;
import org.apache.muse.ws.notification.impl.SubscribeHandler;
import org.apache.muse.ws.resource.WsResource;
import org.apache.muse.ws.resource.lifetime.ScheduledTermination;
import org.apache.muse.ws.resource.lifetime.WsrlConstants;
import org.n52.swe.sas.muse.wsbr.*;
import org.w3c.dom.Element;

/**
 * @author Jan Torben Heuer <jan.heuer@uni-muenster.de>
 * 
 * @see SubscribeHandler
 */
public class PublisherHandler extends AbstractMessageHandler {
	/* automatically generated Logger */@SuppressWarnings("unused")
	private static final Logger LOGGER = Logger.getLogger(PublisherHandler.class.getName());

	public PublisherHandler() {
		super(RegisterPublisherHandlerConstants.ADVERTISE_URI, RegisterPublisherHandlerConstants.ADVERTISE_QNAME);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.apache.muse.core.routing.MessageHandler#fromXML(org.w3c.dom.Element)
	 */
	public Object[] fromXML(Element xml) throws SoapFault {
		Publisher adv = new Publisher(xml);
		return adv.toArray();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.apache.muse.core.routing.MessageHandler#toXML(java.lang.Object)
	 */
	public Element toXML(Object result) throws SoapFault {
		WsResource resource = (WsResource) result;

		PublisherEndpoint endpointInstance = (PublisherEndpoint) resource.getCapability(IPublisherEndpoint.NAMESPACE_URI);

		/* add termination time to the response, if it exists */
        ScheduledTermination wsrl = (ScheduledTermination)resource.getCapability(WsrlConstants.SCHEDULED_TERMINATION_URI);
        Date time = null;
        
        if (wsrl != null)
            time = wsrl.getTerminationTime();

		PublisherResponse response = new PublisherResponse(endpointInstance,time);
		return response.toXML();
	}
}
