/***************************************************************
Copyright 2007 52North Initiative for Geospatial Open Source Software GmbH

 Author: jtheuer, University of Muenster

 Contact: Andreas Wytzisk, 
 52North Initiative for Geospatial Open Source SoftwareGmbH, 
 Martin-Luther-King-Weg 24,
 48155 Muenster, Germany, 
 info@52north.org

 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
 version 2 as published by the Free Software Foundation.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; even without the implied WARRANTY OF
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program (see gnu-gpl v2.txt). If not, write to
 the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 Boston, MA 02111-1307, USA or visit the Free
 Software Foundations web page, http://www.fsf.org.

 ***************************************************************/

package org.n52.swe.sas.muse.wsbr.soaphandler;

import java.util.Date;
import java.util.logging.Logger;

import org.apache.muse.util.xml.XmlUtils;
import org.apache.muse.ws.addressing.EndpointReference;
import org.apache.muse.ws.notification.WsnConstants;
import org.apache.muse.ws.notification.impl.SimpleNotificationProducer;
import org.n52.swe.sas.muse.wsbr.*;
import org.w3c.dom.Document;
import org.w3c.dom.Element;

/**
 * @author Jan Torben Heuer <jan.heuer@uni-muenster.de>
 * @see SimpleNotificationProducer for developer help
 */
public class PublisherResponse {
	/* automatically generated Logger */@SuppressWarnings("unused")
	private static final Logger LOGGER = Logger.getLogger(PublisherResponse.class.getName());
	private PublisherEndpoint endpoint;
	private Date time;

	public PublisherResponse(PublisherEndpoint endpoint, Date time) {
		this.endpoint = endpoint;
		this.time = time;
	}
	
	public Element toXML() {
		return toXML(XmlUtils.EMPTY_DOC);
	}

	public Element toXML(Document doc) {
		Element root = XmlUtils.createElement(doc, RegisterPublisherHandlerConstants.ADVERTISE_RESPONSE_QNAME);
		
		final EndpointReference notificationProducerReference = endpoint.getNotificationProducerReference();
		final EndpointReference producerReference =endpoint.getPublisherReference();
		
		if(time != null) {
			XmlUtils.setElement(root, WsnConstants.TERMINATION_TIME_QNAME, time);
		}
		
		XmlUtils.setElement(root, WsbrConstants.PUBLISHER_REFERENCE_QNAME, producerReference.toXML());
		XmlUtils.setElement(root, WsnConstants.CONSUMER_QNAME, notificationProducerReference.toXML());
		
		return root;
	}
}
