package org.n52.swe.sas.muse.wsn;

import java.util.*;

import javax.xml.namespace.QName;

import net.opengis.eml.x001.EMLDocument.EML;

import org.apache.muse.ws.addressing.EndpointReference;
import org.apache.muse.ws.addressing.soap.SoapFault;
import org.apache.muse.ws.notification.*;
import org.apache.muse.ws.notification.faults.*;
import org.apache.muse.ws.notification.impl.*;
import org.apache.muse.ws.notification.topics.Topic;
import org.apache.muse.ws.resource.WsResource;
import org.n52.swe.sas.muse.filter.*;
import org.n52.swe.sas.muse.filter.engine.FilterEngine;
import org.n52.swe.sas.muse.filter.engine.IFilterEngine;
import org.n52.swe.sas.muse.unitconversion.IUnitConverter;
import org.n52.swe.sas.muse.unitconversion.SESUnitConverter;
import org.n52.swe.sas.muse.util.FilterListener;
import org.n52.swe.sas.muse.wsbr.PublisherEndpoint;
import org.n52.swe.sas.muse.wsbr.SesTopicFactory;
import org.w3c.dom.Element;


/**
 * 
 * @author Matthes Rieke <m.rieke@uni-muenster.de>
 *
 */
public class SESNotificationProducer extends SimpleNotificationProducer {

	
	private IFilterEngine _filterEngine;
	private IUnitConverter _unitConverter;

	@SuppressWarnings("unchecked")
	@Override
	public void publish(QName topicName, Element[] content) throws SoapFault {
        //
        // construct the message/payload
        //
        NotificationMessage message = createNotificationMessage();
        
        for (int n = 0; n < content.length; ++n)
            message.addMessageContent(content[n]);
        
        message.setTopic(topicName);
        
        //
        // give the message to each subscription so it can decide if it 
        // should be sent or not
        //
        
        
        /*
         * 
         * check if SubscriptionManager without SESConstraintFilter exists
         * and forward message to them.
         * 
         */
        
        Iterator i = getSubscriptions().iterator();
        
        while (i.hasNext()) {
            SESSubscriptionManager sub = (SESSubscriptionManager)i.next();
            if (!sub.isHasConstraintFilter()) {
            	sub.publish(message);
            }
        }
        
        /*
         * 
         * Entrancepoint for the Esper engine
         * 
         * (normal SubscriptionManager call is not needed any longer)
         */
        
        _filterEngine.filter(message);
        
        /*
         * 
         * End of entrancepoint for the Esper engine
         * 
         */
        
        //
        // if a topic was used, record the message as the 'current' message 
        // for the topic (will be returned by getCurrentMessage())
        //
        if (topicName != null)
        {
            Topic topic = getTopic(topicName);
            if (topic != null)
            	topic.setCurrentMessage(message);
        }
    }

	
	@SuppressWarnings("unchecked")
	@Override
	public WsResource subscribe(EndpointReference er, Filter filter, Date date,
			Policy policy) throws TopicNotSupportedFault,
			UnacceptableInitialTerminationTimeFault,
			SubscribeCreationFailedFault {
		
		getLog().info("subscribing... "+ er);
		
		//check if we have a SESConstraintFilter
		if (filter instanceof FilterCollection) {
			SESConstraintFilter sesConstraint = null;
			FilterCollection filterColl = (FilterCollection) filter;
			Collection<Filter> filters = filterColl.getFilters();
			
			SESFilterCollection sesFilterColl = new SESFilterCollection();
			
			for (Filter f : filters) {
				if (f instanceof SESConstraintFilter) {
					//add this one later
					sesConstraint = (SESConstraintFilter) f;
				} else {
					sesFilterColl.addFilter(f);
				}
			}
			
			if (sesConstraint != null) {
				getLog().info("got new SESConstraintFilter..");
				
				/*
				 * check for FilterType or EML
				 * if FilterType -> build static EMLDocument
				 */
				EML eml = null;
				if (sesConstraint.getFilter() != null) {
					eml = StaticEMLDocument.getWrapperDocument(sesConstraint.getFilter());
				} else if (sesConstraint.getEml() != null) {
					eml = sesConstraint.getEml();
				}
				
				SESConstraintFilter scf = new SESConstraintFilter(eml);
				
				sesFilterColl.addFilter(scf);
			} else {
				getLog().info("subscription without SESConstraintFilter..");
			}
			
			//all filters are now in SESFilterCollection
			filter = sesFilterColl;
		}
		
		WsResource result = super.subscribe(er, filter, date, policy);
		SESSubscriptionManager subMgr = (SESSubscriptionManager)
				result.getCapability(WsnConstants.SUBSCRIPTION_MGR_URI);		
		
		//check for SESConstraintFilter and registers at IFilterEngine
		if (subMgr.isHasConstraintFilter()) {
			try {
				_filterEngine.registerFilter(subMgr);
			} catch (Exception e) {
				throw new SubscribeCreationFailedFault(e.getMessage());
			}
		}
		
		return result;
	}

	@Override
	public void initialize() throws SoapFault {
		
		getLog().info("initialising SESNotificationProducer..");
		_unitConverter = new SESUnitConverter(getLog());
		FilterEngine filterEngine = new FilterEngine(_unitConverter, getLog());
		_filterEngine = filterEngine;
		
		//add the SESConstraintFilter
		FilterFactory.getInstance().addHandler(new SESConstraintFilterHandler());
		
		/* let the filter listen for new sensor registrations */
		getResource().getResourceManager().addListener(new FilterListener<PublisherEndpoint>(PublisherEndpoint.class,filterEngine));
		super.initialize();
		
		/* add default topics */
		SesTopicFactory.addDefaultTopics(this);
	}

	@Override
	public void initializeCompleted() throws SoapFault {
		
		getLog().info("initialisation of SESNotificationProducer done!");
		
		super.initializeCompleted();
		
		NotificationConsumer consumer = (NotificationConsumer)
		getResource().getCapability(WsnConstants.CONSUMER_URI);
		
		/*
		 * add Listener to Consumer (needed for notify messages)
		 */
		consumer.addMessageListener(new SESMessageListener(this));
	}
	
	

}
