package org.n52.swe.sas.muse.wsn;


import java.net.URI;
import java.net.URISyntaxException;

import org.apache.muse.util.LoggingUtils;
import org.apache.muse.util.messages.Messages;
import org.apache.muse.util.messages.MessagesFactory;
import org.apache.muse.ws.addressing.EndpointReference;
import org.apache.muse.ws.addressing.soap.SoapFault;
import org.apache.muse.ws.notification.Filter;
import org.apache.muse.ws.notification.NotificationMessage;
import org.apache.muse.ws.notification.impl.SimpleSubscriptionManager;
import org.apache.muse.ws.notification.remote.NotificationConsumerClient;
import org.n52.swe.sas.muse.filter.SESFilterCollection;


/**
 * 
 * @author Matthes Rieke <m.rieke@uni-muenster.de>
 *
 */
public class SESSubscriptionManager extends SimpleSubscriptionManager {

	
	 /* needed by the PublisherEndpoint to retrieve the instance */
	public static final String CONTEXT_PATH = "SubscriptionManagerContextPath";
	
	//
	// Used to lookup all exception messages
	//
	@SuppressWarnings("unused")
	private static Messages _MESSAGES = MessagesFactory.get(SimpleSubscriptionManager.class);
	
	/**
	 * Flag is true if the SESSubscriptionManager has a
	 * SESConstraintFilter. 
	 */
	private boolean hasConstraintFilter = false;
	
	/**
	 * Flag is true if the SESSubscriptionManager has a
	 * SESConstraintFilter. 
	 */
	public boolean isHasConstraintFilter() {
		return hasConstraintFilter;
	}



	@Override
	public void initialize() throws SoapFault {

		getLog().info("initialising SESSubscriptionManager..");

		super.initialize();
	}



	@Override
	public EndpointReference getConsumerReference() {
		if (super.getConsumerReference() == null) {
			try {
				return new EndpointReference(new URI("http://google.de"));
			} catch (URISyntaxException e) {
				e.printStackTrace();
			}
			return null;
		}
		
		else {
			return super.getConsumerReference();
		}
	}



	@Override
	public void setFilter(Filter filter) {
		if (filter instanceof SESFilterCollection) {
			SESFilterCollection filterColl = (SESFilterCollection) filter;
			if (filterColl.getConstraintFilter() != null) hasConstraintFilter = true;
		}
		super.setFilter(filter);
	}



	@Override
	public void publish(NotificationMessage message) {

		//check the filters except SESConstraintFilter
		boolean accepted = getFilter().accepts(message);
		
		//just check if paused, do not check the filters - this is done before
		if (isPaused() || !accepted)
			return;

		//
		// set EPRs for this publication
		//
		message.setProducerReference(getProducerReference());
		message.setSubscriptionReference(getResource().getEndpointReference());

		boolean success = false;
		int tries = getNumberOfTries();

		for (int n = 0; n < tries && !success; ++n)
		{
			getLog().info("sending message... "+n +"... to:\n"+ getConsumerReference().toString());
			
			
			try
			{
				NotificationConsumerClient client = getConsumerClient();
				client.notify(new NotificationMessage[]{ message });

				success = true;
			}

			catch (SoapFault error)
			{
				LoggingUtils.logError(getLog(), error);

				Object[] filler = { error.getMessage() };
				getLog().info(_MESSAGES.get("LastPublishFailed", filler));
			}
		}

		try
		{
			if (!success && isDestroyedOnFailure())
				getResource().shutdown();
		}

		catch (SoapFault error)
		{
			LoggingUtils.logError(getLog(), error);
		}
	}



}