package org.n52.swe.sas.muse;

import org.apache.muse.util.messages.Messages;
import org.apache.muse.util.messages.MessagesFactory;
import org.apache.muse.util.xml.XmlUtils;
import org.apache.muse.ws.addressing.EndpointReference;
import org.apache.muse.ws.addressing.soap.*;
import org.w3c.dom.Document;
import org.w3c.dom.Element;

public class InlineSoapClient extends SimpleSoapClient {
	private InlineIsolationLayer layer;
	 
	public InlineSoapClient(InlineEnvironment environment, InlineIsolationLayer isoLayer) {
		 layer = isoLayer;
	 }
	//
    // Used to lookup all exception messages
    //
    private static Messages _MESSAGES = MessagesFactory.get(SimpleSoapClient.class);
    private static final Element[] _EMPTY_ARRAY = new Element[0];
 
	// uses template code from simplesoapclient
	@Override
	public Element[] send(EndpointReference src, 
            EndpointReference dest,
            String wsaAction, 
            Element[] body, 
            Element[] extraHeaders)
	{
		if (dest == null)
		throw new NullPointerException(_MESSAGES.get("NullDestinationEPR"));
		
		if (wsaAction == null)
		throw new NullPointerException(_MESSAGES.get("NullActionURI"));
		
		if (body == null)
		body = _EMPTY_ARRAY;
		
		if (extraHeaders == null)
		extraHeaders = _EMPTY_ARRAY;
		
		//
		// create the request message and turn it into bytes
		//
		Element soapRequest = createMessage(src, dest, wsaAction, body, extraHeaders);
		
//		if (isUsingTrace())
//			trace(soapRequest, false);
		
		Element soapResponse = null;
				
		try
		{
			//
			// send back into muse
			//
			Document reqDoc = soapRequest.getOwnerDocument();
			reqDoc.appendChild(soapRequest); // add it in, as its not done by the createMessage function
			Document responseDoc = layer.handleRequest(reqDoc); 
			
			if (responseDoc  == null) {
				// just never got a response, for some valid reason, which sucks
				// some RT operations return nothing though, isolation layer covers this
				return new Element[]{};
			}
			
			//
			// read in the response and build an XML document from it
			//
			soapResponse = XmlUtils.getFirstElement(responseDoc);
			
		
		}
		
		//
		// handle any other runtime/IO exceptions as best we can
		//
		catch (Throwable error)
		{
			SoapFault soapFault = new SoapFault(error.getMessage(), error);
			return new Element[]{ soapFault.toXML() };
		} finally {
		}
		
//		if (isUsingTrace())
//		trace(soapResponse, true);
		
		//
		// return the elements inside the SOAP body
		//
		Element responseBody = XmlUtils.getElement(soapResponse, SoapConstants.BODY_QNAME);
		return XmlUtils.getAllElements(responseBody);
	}

}
