/***************************************************************
Copyright 2007 52North Initiative for Geospatial Open Source Software GmbH

 Author: jtheuer, University of Muenster

 Contact: Andreas Wytzisk, 
 52North Initiative for Geospatial Open Source SoftwareGmbH, 
 Martin-Luther-King-Weg 24,
 48155 Muenster, Germany, 
 info@52north.org

 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
 version 2 as published by the Free Software Foundation.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; even without the implied WARRANTY OF
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program (see gnu-gpl v2.txt). If not, write to
 the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 Boston, MA 02111-1307, USA or visit the Free
 Software Foundations web page, http://www.fsf.org.

 ***************************************************************/

	
package org.n52.swe.sas.muse;
import java.io.*;
import java.util.Iterator;
import java.util.NoSuchElementException;

import javax.xml.namespace.QName;
import javax.xml.parsers.ParserConfigurationException;

import junit.framework.TestCase;

import org.apache.commons.io.IOUtils;
import org.apache.muse.core.Resource;
import org.apache.muse.core.ResourceManager;
import org.apache.muse.core.routing.SimpleResourceRouter;
import org.apache.muse.util.xml.XmlUtils;
import org.apache.muse.ws.addressing.EndpointReference;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.xml.sax.SAXException;

/**
 * @author Jan Torben Heuer <jan.heuer@uni-muenster.de>
 *
 */
public abstract class MuseTestCase extends TestCase {

	protected InlineIsolationLayer iso;

	/**
	 * @param name
	 */
	public MuseTestCase(String name) {
		super(name);
	}

	/* (non-Javadoc)
	 * @see junit.framework.TestCase#setUp()
	 */
	@Override
	protected void setUp() throws Exception {
		super.setUp();
		iso = new InlineIsolationLayer();
		iso.initialize();
		File dir = iso.getEnvironment().getRealDirectory();
		
		/* delete temporal files */
		File routerEntries = new File(dir.getAbsolutePath()+File.separator+"router-entries");
		if(routerEntries.exists()) {
			File prm = new File(routerEntries+File.separator+"SesPRMContextPath");
			if(prm.exists()) {
				for(File res: prm.listFiles()) {
					res.delete();
				}
			}
		}

	}

	/* (non-Javadoc)
	 * @see junit.framework.TestCase#tearDown()
	 */
	@Override
	protected void tearDown() throws Exception {
//		if(!iso.hasBeenShutdown()){
//			iso.shutdown();
//		}
	}
	
	protected String readResource(String name) throws IOException {
		InputStream stream = getClass().getResourceAsStream(name);
		if(stream==null) {
			throw new IOException("stream is null");
		}
		StringWriter output = new StringWriter();
		IOUtils.copy(stream, output);
		return output.toString();
	}
	
	protected Document readResourceAsDom(String name) throws SAXException, IOException, ParserConfigurationException {
		return XmlUtils.createDocument(getClass().getResourceAsStream(name));
	}
	
	/**
	 * Tests if the supplied {@link Element} subtree contains the {@link QName} element.
	 * @param context The subtree
	 * @param qname The element
	 * @return true if there are one or more matches
	 */
	protected void assertContainsElement(Element context, QName qname) {
		if(context == null || qname == null) {
			throw new IllegalArgumentException("parameters must not be null!");
		}
		Element[] result = XmlUtils.findInSubTree(context, qname);
		assertTrue(result != null && result.length > 0);
	}
	
	/**
	 * Tests if the supplied {@link Element} subtree contains the {@link QName} element.
	 * @param context The subtree
	 * @param qname The element
	 * @return true if there are one or more matches
	 */
	protected void assertContainsElement(Document context, QName qname) {
		assertContainsElement(context.getDocumentElement(), qname);
	}
	
	protected void assertNotContainsElement(Element context, QName qname) {
		Element[] result = XmlUtils.findInSubTree(context, qname);
		assertTrue(result == null || result.length == 0);
	}
	
	/**
	 * Iterates over all available Resources for the first one that matches the given classtype
	 * @param capabilityClass
	 * @return
	 */
	protected Resource getResourceOf(Class<?> capabilityClass) {
		ResourceManager manager = ( (SimpleResourceRouter) iso.getRouter() ) .getResourceManager();
		String contextPath = manager.getResourceContextPath(capabilityClass);  //fullURL?  iso.getRouter().getEnvironment().getDefaultURI() +
		Iterator<?> eprs = manager.getResourceEPRs(contextPath);

		/* check if there is at least one matching resource */
		if(eprs.hasNext()) {
			return getResourceOf((EndpointReference)eprs.next());
		} else {
			throw new NoSuchElementException(contextPath + " is not available. Available Resources are: " + manager.getResourceContextPaths());
		}
	}
	
	/**
	 * returns the Resource associated with the given endpoint
	 * @param endpoint
	 * @return
	 */
	protected Resource getResourceOf(EndpointReference endpoint) {
		ResourceManager manager = ( (SimpleResourceRouter) iso.getRouter() ) .getResourceManager();
		return manager.getResource(endpoint);
	}
}
