/***************************************************************
Copyright 2007 52North Initiative for Geospatial Open Source Software GmbH

 Author: jtheuer, University of Muenster

 Contact: Andreas Wytzisk, 
 52North Initiative for Geospatial Open Source SoftwareGmbH, 
 Martin-Luther-King-Weg 24,
 48155 Muenster, Germany, 
 info@52north.org

 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
 version 2 as published by the Free Software Foundation.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; even without the implied WARRANTY OF
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program (see gnu-gpl v2.txt). If not, write to
 the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 Boston, MA 02111-1307, USA or visit the Free
 Software Foundations web page, http://www.fsf.org.

 ***************************************************************/

package org.n52.swe.sas.muse.soapclient;

import java.io.*;
import java.net.*;
import java.util.logging.Logger;

import org.apache.commons.io.IOUtils;

public class SoapClient {

	/* automatically generated Logger */@SuppressWarnings("unused")
	private static final Logger LOGGER = Logger.getLogger(SoapClient.class.getName());
	
	private URL url;

	/**
	 * Creates a new {@link SoapClient} that connects to the given url.
	 * All requests are parsed for the \@url\@ string and replaced by the url.
	 */
	public SoapClient(URL server) {
		url = server;
	}

	public String request(InputStream stream) throws IOException {
		return request(IOUtils.toString(stream));
	}
	
	public String request(String request) throws IOException {

			request = request.replace("@url@", url.toExternalForm());
			
			/* hack around the broken mini-servlet implementation */
			URLConnection uc = url.openConnection();
			HttpURLConnection connection = (HttpURLConnection) uc;

			connection.setDoOutput(true);
			connection.setDoInput(true);
			connection.setRequestMethod("POST");
			//connection.setRequestProperty("SOAPAction", SOAP_ACTION);

			LOGGER.info(request);
			
			OutputStream out = connection.getOutputStream();
			Writer wout = new OutputStreamWriter(out);

			wout.write(request);

			wout.flush();
			wout.close();

			InputStream in = connection.getInputStream();

			return IOUtils.toString(in);
	} 
}
