/***************************************************************
Copyright 2007 52North Initiative for Geospatial Open Source Software GmbH

 Author: jtheuer, University of Muenster

 Contact: Andreas Wytzisk, 
 52North Initiative for Geospatial Open Source SoftwareGmbH, 
 Martin-Luther-King-Weg 24,
 48155 Muenster, Germany, 
 info@52north.org

 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
 version 2 as published by the Free Software Foundation.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; even without the implied WARRANTY OF
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program (see gnu-gpl v2.txt). If not, write to
 the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 Boston, MA 02111-1307, USA or visit the Free
 Software Foundations web page, http://www.fsf.org.

 ***************************************************************/

package org.n52.swe.sas.muse.util;

import java.net.URI;
import java.util.logging.Logger;

import org.apache.muse.core.SimpleResource;
import org.apache.muse.ws.addressing.EndpointReference;
import org.apache.muse.ws.addressing.soap.SoapFault;
import org.n52.swe.sas.muse.MuseTestCase;

/**
 * @author Jan Torben Heuer <jan.heuer@uni-muenster.de>
 * 
 */
public class FilterListenerTest extends MuseTestCase implements AdvancedResourceManagerListener<Resource1> {
	/* automatically generated Logger */@SuppressWarnings("unused")
	private static final Logger LOGGER = Logger.getLogger(FilterListener.class.getName());
	private FilterListener<Resource1> fl;
	private Resource1 res1;
	private EndpointReference epr1;
	private EndpointReference epr2;
	private Resource2 res2;
	private int listenerFiredCount=0;

	/**
	 * @param name
	 */
	public FilterListenerTest(String name) {
		super(name);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.n52.swe.sas.muse.MuseTestCase#setUp()
	 */
	@Override
	protected void setUp() throws Exception {
		fl = new FilterListener<Resource1>(Resource1.class, this);
		epr1 = new EndpointReference(new URI("http://example.com/resource1"));
		epr2 = new EndpointReference(new URI("http://example.com/resource2"));
		res1 = new Resource1();
		res2 = new Resource2();

	}

	/**
	 * Test method for
	 * {@link org.n52.swe.sas.muse.util.FilterListener#resourceAdded(org.apache.muse.ws.addressing.EndpointReference, org.apache.muse.core.Resource)}.
	 * @throws SoapFault 
	 */
	public void testResourceAdded() throws SoapFault {
		fl.resourceAdded(epr1, res1);
		fl.resourceRemoved(epr1);
		fl.resourceAdded(epr2, res2);
		fl.resourceRemoved(epr2);
		assertTrue(listenerFiredCount==2);
	}

	/* (non-Javadoc)
	 * @see org.n52.swe.sas.muse.util.AdvancedResourceManagerListener#resourceAdded(org.apache.muse.ws.addressing.EndpointReference, java.lang.Object)
	 */
	public void resourceAdded(EndpointReference epr, Resource1 resource) throws SoapFault {
		assertTrue(resource.equals(res1));
		listenerFiredCount++;
	}

	/* (non-Javadoc)
	 * @see org.n52.swe.sas.muse.util.AdvancedResourceManagerListener#resourceRemoved(org.apache.muse.ws.addressing.EndpointReference, java.lang.Object)
	 */
	public void resourceRemoved(EndpointReference epr, Resource1 resource) throws SoapFault {
		assertTrue(resource.equals(res1));
		listenerFiredCount++;
	}

}

/* dummy resources */
final class Resource1 extends SimpleResource {

}

final class Resource2 extends SimpleResource {

}
