/***************************************************************
Copyright 2007 52North Initiative for Geospatial Open Source Software GmbH

 Author: jtheuer, University of Muenster

 Contact: Andreas Wytzisk, 
 52North Initiative for Geospatial Open Source SoftwareGmbH, 
 Martin-Luther-King-Weg 24,
 48155 Muenster, Germany, 
 info@52north.org

 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
 version 2 as published by the Free Software Foundation.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; even without the implied WARRANTY OF
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program (see gnu-gpl v2.txt). If not, write to
 the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 Boston, MA 02111-1307, USA or visit the Free
 Software Foundations web page, http://www.fsf.org.

 ***************************************************************/

	
package org.n52.swe.sas.muse.wsbr;
import java.util.Calendar;
import java.util.GregorianCalendar;
import java.util.logging.Logger;

import org.apache.muse.core.Resource;
import org.apache.muse.util.xml.XmlUtils;
import org.apache.muse.util.xml.XsdUtils;
import org.apache.muse.ws.notification.WsnConstants;
import org.apache.muse.ws.wsdl.WsdlUtils;
import org.n52.swe.sas.muse.MuseTestCase;
import org.n52.swe.sas.muse.wsrf.WsrfRlConstants;
import org.w3c.dom.Document;
import org.w3c.dom.Element;

/**
 * @author Jan Torben Heuer <jan.heuer@uni-muenster.de>
 *
 * test the {@link RegisterPublisher} class
 */
public class RegisterPublisherTestCase extends MuseTestCase {
	/* automatically generated Logger */@SuppressWarnings("unused")
	private static final Logger LOGGER = Logger.getLogger(RegisterPublisherTestCase.class.getName());

	/**
	 * @param name
	 */
	public RegisterPublisherTestCase(String name) {
		super(name);
	}

	/* (non-Javadoc)
	 * @see org.n52.swe.sas.muse.MuseTestCase#setUp()
	 */
	@Override
	protected void setUp() throws Exception {
		super.setUp();
	}
	

	public void testRegister() throws Exception {
		Resource resource = getResourceOf(RegisterPublisher.class);
		RegisterPublisher rp = (RegisterPublisher) resource.getCapability(WsbrConstants.NAMESPACE_URI);
		
		/* check if rp is empty */
		assertTrue("Expected that no PRM instances exist", rp.getPublisherEndpoints().size() == 0);
		
		Document result = iso.handleRequest(readResourceAsDom("/testRegisterPublisher.xml"));

		
		//uncomment to show results
		LOGGER.info(XmlUtils.toString(result));
		
		/* result is correct */
		assertContainsElement(result.getDocumentElement(), WsbrConstants.PUBLISHER_REFERENCE_QNAME);
		assertContainsElement(result.getDocumentElement(), WsnConstants.TERMINATION_TIME_QNAME);

		
		/* check if rp contains the new publicationendpoint */
		assertTrue(rp.getPublisherEndpoints().size() == 1);
	}
	
	public void testExpire() throws Exception {
		Resource resource = getResourceOf(RegisterPublisher.class);
		RegisterPublisher rp = (RegisterPublisher) resource.getCapability(WsbrConstants.NAMESPACE_URI);
		int count = rp.getPublisherEndpoints().size();
		
		GregorianCalendar inAfewSeconds = new GregorianCalendar();
		inAfewSeconds.add(Calendar.SECOND, 3);

		Document dom = readResourceAsDom("/testRegisterPublisher.xml");
		Element terminationTime = XmlUtils.findFirstInSubTree(dom.getDocumentElement(), WsrfRlConstants.REQUESTED_LIFETIME_DURATION);
		XmlUtils.setElementText(terminationTime,XsdUtils.getLocalTimeString(inAfewSeconds.getTime()));
		iso.handleRequest(dom);
		assertEquals(count + 1,rp.getPublisherEndpoints().size());
		
		/* wait for termination */
		Thread.sleep(4000);
		assertEquals(count,rp.getPublisherEndpoints().size());
		
	}
	
	public final void testDestroy() throws Exception, Exception, Exception {
		Resource resource = getResourceOf(RegisterPublisher.class);
		RegisterPublisher rp = (RegisterPublisher) resource.getCapability(WsbrConstants.NAMESPACE_URI);
		
		/* check if rp is empty */
		int count = rp.getPublisherEndpoints().size();
		Document result = iso.handleRequest(readResourceAsDom("/testRegisterPublisher.xml"));
		
		/* check if there is one PublisherEndpoint */
		assertTrue(rp.getPublisherEndpoints().size() == count + 1);
		PublisherEndpoint pe = rp.getPublisherEndpoints().iterator().next();
		
		Element destroyed = pe.destroyRegistration(null);
		
		/* check if rp is empty again */
		assertTrue("Did not find the correct amount of PRM instances after destroying", rp.getPublisherEndpoints().size() == count);
		
	}
}
